****************************************************************************************
*                                                                                      *
*  AppleIIgsKarate.Game.s :  Game core functions.                                      *
*                                                                                      *
****************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  Oct 2022  *
****************************************************************************************

*----------   Global Equates : Firmware / Hardware / GS/OS   ---------
KBDREAD             =          $00C000              ; Read Keyboard
CLR80VID            =          $00C00C              ; Disable 80 Column display
SET80VID            =          $00C00D              ; Enable 80 Column display
KBDSTRB             =          $00C010              ; Keyboard Strobe
RDVBLBAR            =          $00C019              ; VBL Status (Bit #7=0 if VBL)
RD80VID             =          $00C01F              ; 80 column status (Bit #7=1 if 80 column)
TBCOLOR             =          $00C022              ; Text / Background color
KEYMODREG           =          $00C025              ; Special Keys (Shift, Control, Apple...)
NEWVIDEO            =          $00C029              ; Apple IIgs Video Mode
VERTCNT             =          $00C02E              ; Video Controler Vertical Counter
HORIZCNT            =          $00C02F              ; Video Controler Horizontal Counter
CLOCKCTL            =          $00C034              ; Border Color + Clock
SHADOW              =          $00C035              ; Shadowing Register
SOUNDCTL            =          $E1C03C              ; Ensoniq 5503 Control Register
SOUNDDATA           =          $E1C03D              ; Ensoniq 5503 Data Register
SOUNDADRL           =          $E1C03E              ; Ensoniq 5503 Address Low
SOUNDADRH           =          $E1C03F              ; Ensoniq 5503 Address High
BUTN0               =          $00C061              ; Open Apple Key = Joystick Button 0
TEXTPAGE1           =          $E00400              ; Text Page #1 $0400-$07FF
GSOS                =          $E100A8              ; GS/OS Entry Point
CTRLPNLSNDVOL       =          $E100CA              ; Control Panel Sound Volume

*----------   Merlin 16+ Directives   ---------

                    mx         %00                  ; Assemble in 16 bit

                    rel                             ; Build a relocated S16 file
                    dsk        Game.l               ; Name of the Segment

*----------   External Labels from AppleIIgsKarate.s Segment  ---------
NewHighScoreScreen  EXT                             ; Insert new high score into table
ClearKDBStrobe      EXT                             ; Clear Keyboard Strobe
FadeOut             EXT                             ; FadeOut Screen
SetBorderColor      EXT                             ; Change border color
UnpackLZ4           EXT                             ; Unpack LZ4 data buffer
Buffer_Palette      EXT                             ; 16 Palettes buffer
DisplayString       EXT                             ; Display a String on the Screen
FadeIN              EXT                             ; Fade In Screen from Black to Picture
SS_PLAYER_1         EXT                             ; Player #1 Control = 1:Joystick, 2:Joypad ,3:Keyboard, 4:Computer
SS_PLAYER_2         EXT                             ; Player #1 Control = 1:Joystick, 2:Joypad ,3:Keyboard, 4:Computer
LoadFile            EXT                             ; Load one File into memory
Score_Bitmap_Tab    EXT                             ; Score Bitmap Table
CopyOneBitmap       EXT                             ; Copy one Bitmap to Screen
Score_0             EXT                             ; 0 Bitmap                   
WaitForSpot         EXT                             ; Wait for the Video Spot to cross a screen line    
ReadVideoSpot       EXT                             ; Get Video Spot position
WaitForKey          EXT                             ; Wait for a Key press
PauseGame           EXT                             ; Pause Audio + Video waiting for a Key press  
SetOscVolume        EXT                             ; Define Oscillator Volume
EnableDisableAudio  EXT                             ; Swith On/Off Audio
MuteUnmuteAudio     EXT                             ; Mute / Unmute Audio
CopySoundDataToRAM  EXT                             ; Copy Sound Data to Sound RAM
CopyData2BankToDOC  EXT                             ; Copy Sound Data from Data2Bank to DOC RAM
PlayLoopSound       EXT                             ; Start Sound in Swap Mode xKB+xKB
PlayOneShotSound    EXT                             ; Start Sound in One Shot Mode
FadeOutSound        EXT                             ; Decrease Sound Volume slowly
StopSound           EXT                             ; Stop Sound (all 32 oscillators)
IncBorderColor      EXT                             ; Increment Border Color  
HasAppleJoystick    EXT                             ; Is Apple Joystick connected ?
GetJoystickInput    EXT                             ; Get Apple Joystick Inputs
GetJoypadSlot       EXT                             ; Get Joypad Slot #                 
GetJoypad1Input     EXT                             ; Get Joypad #1 Inputs
GetJoypad2Input     EXT                             ; Get Joypad #2 Inputs
GetKeyboardInput    EXT                             ; Get Keyboard Inputs
GetRandomNumber     EXT                             ; Get Random Number in range [1-100]
             
*----------   External Labels from AppleIIgsKarate.Sprite*.s Segment  ---------
DrawSpriteBank1     EXT                             ; Compiled Sprites Bank #1
DrawSpriteBank2     EXT                             ; Compiled Sprites Bank #2
DrawSpriteBank3     EXT                             ; Compiled Sprites Bank #3
DrawSpriteBank4     EXT                             ; Compiled Sprites Bank #4
      
*--------  AllocProgramMemory variables  --------
GameBankTable       ENT
LoadUnpackBank      HEX        0000                 ; Load / Unpack Bank   00XX (0000-7FFF : Load / 8000-FFFF : Unpack)
Data1Bank           HEX        0000                 ; Generic Data Bank #1 + LZ4 Game Levels Background Data
Data2Bank           HEX        0000                 ; Generic Data Bank #2 + Game Sound / Music Data
Data3Bank           HEX        0000                 ; Generic Data Bank #3 + Log/Stat Storage
*-
EraseCode1Bank      HEX        0000                 ; Erase Code Bank #1
EraseCode2Bank      HEX        0000                 ; Erase Code Bank #2
EraseCode3Bank      HEX        0000                 ; Erase Code Bank #3
      
      
************************************
*  PlayGame() :  Start a new Game. * 
************************************
PlayGame            ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
PG_RandomInit       JSR        RandomInitialize     ; Initialize Random generator
                    JSL        HasAppleJoystick     ; Detect Apple Analog Joystick
                    STA        PG_Joystick
*--
PG_OneTimeInit      LDA        #$0000               ; Run some one-time initialization
                    BNE        PlayGame_Start 
                    JSL        GetJoypadSlot        ; Get Joypad Slot #
                    STA        PG_JoypadSlot
                    JSR        BuildSequenceWidth   ; Build Sequence Width table                                                         
*- Static
                    JSR        InstallPreComputed   ; Install Pre-Computed data
                    LDA        EraseCode3Bank       ; Use EraseCode3Bank for Collision Data Storage
                    STA        GCS_GetRecvAreaY_1+2
                    STA        GCS_GetRecvAreaY_2+2
                    STA        GCS_GetRecvAreaY_4+2
                    STA        GCS_SearchBegin_3+2
                    STA        GCS_CRLS_Loop+2
                    STA        GCS_CRLS_Process_1+2
                    STA        GCS_CRLS_Process_2+2                    
*- Dynamic
*             JSR        BuildEraseSequence   ; Build Erase Sequence based on Animation Sequence
*             JSR        BuildAllEraseCode    ; Build Erase Code for all registered Erase Sequence
*             JSR        BuildAllCollisition  ; Build Collision Data                    
*-
                    INC        PG_OneTimeInit+1
*--
PlayGame_Start      LDA        #$0001               ; Start Level : 1  ### Debug
                    STA        PG_Level
                    STZ        PG_Score+0           ; Reset Score : 0
                    STZ        PG_Score+2
                    STZ        PG_Score+4
                    STZ        PG_Score+6
                    LDA        #$0003               ; Reset Lives : 3
                    STA        PG_Lives
                    JSR        GetGameType
                    STA        PG_GameType          ; GameType : 0:CompVsComp, 1:PlayerVsComp, 2:PlayerVsPlayer
*--
PlayGame_Level      JSR        PlayLevel            ; Play one Level
                    LDX        PG_GameType
                    CPX        #$0000               ; CompVsComp
                    BEQ        PlayGame_End
                    CPX        #$0002               ; PlayerVsPlayer
                    BEQ        PlayGame_End         
                    CMP        #$001B               ; ErrorCode = ESC
                    BEQ        PlayGame_End
                    CMP        #$0000               ; Win = Next Level
                    BEQ        PlayGame_Win
*--
PlayGame_Loose      LDA        PG_Lives             ; End of 3 lives ?
                    CMP        #$0001
                    BEQ        PlayGame_HighScore
                    DEC        PG_Lives
                    BRA        PlayGame_Level       ; Play Same level again        
*--
PlayGame_Win        INC        PG_Level             ; Play Next Level
                    BRA        PlayGame_Level
*--
PlayGame_HighScore  LDA        #PG_Score            ; Check if we have a new High Score
                    LDX        #>PG_Score
                    JSL        NewHighScoreScreen
*--
PlayGame_End        PLB
                    RTL
                    
*----------   PlayGame Variables   ---------                     
PG_Level            ENT
                    HEX        0000                 ; 01-99 [0-9]
PG_Score            ENT
                    HEX        0000,0000,0000,0000  ; [0-9]
PG_Lives            HEX        0000                 ; 3
PG_GameType         HEX        0000                 ; 0:CompVsComp, 1:PlayerVsComp, 2:PlayerVsPlayer 
PG_GameOpponent     HEX        0000                 ; 0:Vase(Lvl4|8), 1:Lvl<=3|PlVsPl, 2:Lvl>=5 
PG_JoypadSlot       HEX        0000                 ; Joypad Slot #
PG_Joystick         HEX        0000                 ; Joystick 


****************************************************
*  PlayLevel() :  Start a new Level (OUT:A=Error). * 
****************************************************                    
PlayLevel           LDA        PG_Level             ; Turn Game Level (01-99) into Game Index (1-6)
                    JSR        GetGameIndex
                    STA        PL_GameIndex
*--
                    JSR        GetGameOpponent      ; Get Opponent # : 0-2 
                    STA        PG_GameOpponent      
*--
PL_LoadMusic        LDA        PL_GameIndex         ; Load Data/GameMusicX.bin file   
                    JSR        LoadGameMusic                          
*--
PL_LoadBackGround   LDA        PL_GameIndex
                    JSR        LoadGameBackground   ; Load Data/GameBackgX.bin file + Copy to LoadUnpackBank                                     
*----
PL_BlackScreen      JSL        ClearKDBStrobe       ; Clear Keyboard
                    JSL        FadeOut              ; Fadout to Black
                    LDA        #$0000
                    JSL        SetBorderColor
*----
PL_UnpackBg         LDA        PL_GameIndex         ; Uncompress Background
                    DEC
                    ASL
                    TAX
                    LDA        LGB_LZ4Size,X        ; LZ4 Size
                    STA        PL_UnpackBg_Size+1        
                    SEP        #$20                 ; LZ4 Uncompress
                    LDA        LoadUnpackBank+1
                    XBA
                    LDA        LoadUnpackBank+1     ; Src Bank | Dst Bank
                    REP        #$30
                    LDX        #$0080               ; Src Offset | Dst Offset ($80=$8000)
PL_UnpackBg_Size    LDY        #$AAAA               ; PackDataSize
                    JSL        UnpackLZ4
                    CMP        #$8000               ; => Return in A the length of unpacked Data
*--
                    LDA        LoadUnpackBank       ; Copy Pixel + SCB
                    ORA        #$0080
                    STA        PL_UnpackBg_Copy+2
                    LDA        LoadUnpackBank       ; Palette
                    ORA        #$00FE                    
                    STA        PL_UnpackBg_Palette+2
                    LDX        #$7DFE       
PL_UnpackBg_Copy    LDAL       $AA8000,X
                    STAL       $012000,X
                    DEX
                    DEX
                    BPL        PL_UnpackBg_Copy
*--
                    LDA        #$0101               ; Install SCBs
                    LDX        #$0016
PL_SCB              STAL       $019D00,X
                    DEX
                    DEX
                    BPL        PL_SCB
                    LDA        #$0101
                    STAL       $019D00+2
                    STAL       $019D00+14
                    LDA        #$0202
                    STAL       $019D00+4
                    STAL       $019D00+16
                    LDA        #$0302
                    STAL       $019D00+6
                    STAL       $019D00+18
                    LDA        #$0303
                    STAL       $019D00+8
                    STAL       $019D00+20
*--
                    LDX        #$001E               ; Copy Src Palette               
PL_UnpackBg_Palette LDAL       $AAFE00,X
                    STAL       Buffer_Palette,X
                    DEX
                    DEX
                    BPL        PL_UnpackBg_Palette
                    LDX        #$001E               ; Duplicate Src Palette               
PL_Duplicat_Palette LDAL       Buffer_Palette,X
                    STAL       Buffer_Palette+32,X  ; Palette #1
                    STAL       Buffer_Palette+64,X  ; Palette #2
                    STAL       Buffer_Palette+96,X  ; Palette #3  
                    DEX
                    DEX
                    BPL        PL_Duplicat_Palette
                    LDA        #$0E86               ; Light Red
                    STAL       Buffer_Palette+32+30
                    LDA        #$0E00               ; Medium Red
                    STAL       Buffer_Palette+64+30
                    LDA        #$0800               ; Dark Red
                    STAL       Buffer_Palette+96+30
*--
PL_Labels           LDX        #PL_LABEL_ENERGY
                    LDY        #$2150               ; @Dst 32,2
                    JSL        DisplayString
                    LDX        #PL_LABEL_SCORE
                    LDY        #$28CC               ; @Dst 24,14
                    JSL        DisplayString
                    LDX        #PL_LABEL_LIVES
                    LDY        #$290C               ; @Dst 152,14
                    JSL        DisplayString
                    LDX        #PL_LABEL_LEVEL
                    LDY        #$2934               ; @Dst 232,14
                    JSL        DisplayString
*----
                    LDA        #PG_Score            ; Draw Score
                    JSR        DrawScore 
                    LDA        PG_Lives             ; Draw #Lives
                    JSR        DrawLives
                    LDA        PG_Level             ; Draw Level
                    JSR        DrawLevel
*----
PL_LifeBar          STZ        PL_LifeBar_Red1      ; Set Life Bar
                    STZ        PL_LifeBar_Red2
                    LDA        PG_GameOpponent
                    BEQ        PL_LifeBar_End
                    LDA        #$00C8
                    STA        PL_LifeBar_Red1
                    LDA        PG_GameOpponent
                    CMP        #$0002
                    BNE        PL_LifeBar_End
                    LDA        #$00C8
                    STA        PL_LifeBar_Red2                    
PL_LifeBar_End      LDA        #$00C8               ; Player Blue Life
                    STA        PL_LifeBar_Blue
*----
                    LDX        #$0062               ; Player #1 (Blue)
PL_PlayerBar        LDA        #$7777
                    STAL       $012033,X
                    STAL       $0120D3,X
                    STAL       $012173,X
PL_Opponent1Bar     LDA        PG_GameOpponent      ; Red Bar #2 is optionnal 
                    BEQ        PL_PlayerBar_Next
                    LDA        #$3333               ; Opponent #1 (Red)
                    STAL       $0122B3,X
                    STAL       $012353,X
                    STAL       $0123F3,X
PL_Opponent2Bar     LDA        PG_GameOpponent      ; Red Bar #2 is optionnal 
                    CMP        #$0002
                    BNE        PL_PlayerBar_Next
                    LDA        #$3333               ; Opponent #2 (Red)
                    STAL       $012533,X
                    STAL       $0125D3,X
                    STAL       $012673,X
PL_PlayerBar_Next   DEX
                    DEX
                    BPL        PL_PlayerBar
*----
PL_BackupBackgound  SEP        #$20                 ; Backup Backgound to $01
                    LDA        #$0F                 ; Shadowing Off (Bit #321 set)
                    STAL       SHADOW
                    REP        #$30
                    LDA        #$329F               ; #bytes - 1  (160*81)
                    LDX        #$64C0               ; Src
                    LDY        #$2F00               ; Dst
                    MVN        $01,$01
                    PHK
                    PLB
                    SEP        #$20
                    LDA        #$00
                    STAL       SHADOW               ; Shadowing On
                    REP        #$30
*----
                    JSR        ResetSpriteEngine    ; Reset Sprites
                    JSR        InstallLevelSprite   ; Install Sprites for this Level
*----
                    LDAL       #Buffer_Palette      ; Display Screen
                    JSL        FadeIN
*----
                    JSR        StartGameMusic       ; Play Music
*----
                    JSR        SpriteEngine         ; Start Game Animation
                    PHA                             ; Return Code
*----
                    JSR        StopGameMusic        ; Stop Music
                    JSL        FadeOut              ; Fade Out Screen to Black
*--
PL_End              PLA                    
                    RTS

*----------   PlayGame Variables   ---------  
PL_GameIndex        HEX        0000                 ; 1-6
PL_LifeBar_Blue     HEX        0000                 ; Life Bar 200-0
PL_LifeBar_Red1     HEX        0000
PL_LifeBar_Red2     HEX        0000
PL_LABEL_ENERGY     ASC        'ENERGY',00
PL_LABEL_SCORE      ASC        'SCORE',00
PL_LABEL_LIVES      ASC        'LIVES',00
PL_LABEL_LEVEL      ASC        'LEVEL',00
PL_Sound            HEX        0000
                                 

************************************************************
*  InstallLevelSprite() :  Install Sprites for this Level. *
************************************************************
InstallLevelSprite  LDA        PG_GameType          ; Game Type 0:CompVsComp, 1:PlayerVsComp, 2:PlayerVsPlayer
                    BEQ        ILS_Go_CompVsComp
                    CMP        #$0002
                    BEQ        ILS_Go_PlayVsPlay
ILS_1               LDA        PG_Level             ; Player vs Comp
                    CMP        #$0004
                    BEQ        ILS_Lvl4
                    CMP        #$0008
                    BEQ        ILS_Lvl8
                    CMP        #$0005
                    BLT        ILS_Go_PlayerVsComp
                    JMP        ILS_PlayerVs2xComp
ILS_Go_CompVsComp   JMP        ILS_CompVsComp
ILS_Go_PlayVsPlay   JMP        ILS_PlayerVsPlayer
ILS_Go_PlayerVsComp JMP        ILS_PlayerVsComp
*----
ILS_Lvl4            LDX        #$0000               ; Level 4 : Vase x 4
ILS_Lvl4_1          LDA        #$0040               ; SpriteID = 40 : Vase
                    LDY        ILS_Lvl4_Tab,X       ; Sprite X0-Y0 on screen (16,134)
                    JSR        InstallNewSprite
                    LDA        #LevelVaseAIMove     ; Level Vase AI Move Function
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #LevelVaseAIReceive  ; Level Vase AI Receive Function
                    STA        SE_SPRITE_FUNC_RECV,X
                    TXA                             ; Bonus Vase Sequence # *2
                    STA        SE_SPRITE_DATA_1,X
                    LDA        ILS_Lvl4_Tab,X       ; Bonus Vase X0Y0 base
                    STA        SE_SPRITE_DATA_2,X
                    INX
                    INX
                    CPX        #$0008
                    BNE        ILS_Lvl4_1
*-
                    LDA        #$0001               ; SpriteID = BR001 : Player Blue Right Up Stand
                    LDX        #$000E               ; Sprite Index #7 / Z-Buffer value
                    LDY        #$1086               ; Sprite X0-Y0 on screen (16,134)
                    JSR        InstallNewSprite
                    LDA        #PlayerBlueAIMove    ; Human Player Blue #1
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #PlayerBlueAIReceive
                    STA        SE_SPRITE_FUNC_RECV,X
                    JMP        ILS_End
*----                    
ILS_Lvl8            LDX        #$0000               ; Level 8 : Vase x 7
ILS_Lvl8_1          LDA        #$0040               ; SpriteID = 40 : Vase
                    LDY        ILS_Lvl8_Tab,X       ; Sprite X0-Y0 on screen (16,134)
                    JSR        InstallNewSprite
                    LDA        #LevelVaseAIMove     ; Level Vase AI Move Function
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #LevelVaseAIReceive  ; Level Vase AI Receive Function
                    STA        SE_SPRITE_FUNC_RECV,X
                    TXA                             ; Bonus Vase Sequence # *2
                    STA        SE_SPRITE_DATA_1,X
                    LDA        ILS_Lvl8_Tab,X       ; Bonus Vase X0Y0 base
                    STA        SE_SPRITE_DATA_2,X
                    INX
                    INX
                    CPX        #$000E
                    BNE        ILS_Lvl8_1
*-
                    LDA        #$0001               ; SpriteID = BR001 : Player Blue Right Up Stand
                    LDX        #$000E               ; Sprite Index #7 / Z-Buffer value
                    LDY        #$1086               ; Sprite X0-Y0 on screen (16,134)
                    JSR        InstallNewSprite
                    LDA        #PlayerBlueAIMove    ; Human Player Blue #1
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #PlayerBlueAIReceive
                    STA        SE_SPRITE_FUNC_RECV,X
                    JMP        ILS_End
*----
ILS_PlayerVsPlayer  LDA        #$0001               ; SpriteID = BR001 : Player Blue Right Up Stand
                    LDX        #$000E               ; Sprite Index #7 / Z-Buffer value
                    LDY        #$1086               ; Sprite X0-Y0 on screen (16,134)
                    JSR        InstallNewSprite
                    LDA        #PlayerBlueAIMove    ; Human Player Blue #1
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #PlayerBlueAIReceive
                    STA        SE_SPRITE_FUNC_RECV,X
*-
                    LDA        #$0062               ; SpriteID = RL062 : Player Red Left Up Stand
                    LDX        #$0010               ; Sprite Index #8 / Z-Buffer value
                    LDY        #$8086               ; Sprite X0-Y0 on screen (128,134)
                    JSR        InstallNewSprite
                    LDA        #PlayerRedAIMove     ; Human Player Red #2
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #PlayerRedAIReceive
                    STA        SE_SPRITE_FUNC_RECV,X
                    JMP        ILS_End
*----
ILS_PlayerVsComp    LDA        #$0001               ; SpriteID = BR001 : Player Blue Right Up Stand
                    LDX        #$000E               ; Sprite Index #7 / Z-Buffer value
                    LDY        #$1086               ; Sprite X0-Y0 on screen (16,134)
                    JSR        InstallNewSprite
                    LDA        #PlayerBlueAIMove    ; Human Player Blue #1
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #PlayerBlueAIReceive
                    STA        SE_SPRITE_FUNC_RECV,X
*-
                    LDA        #$0062               ; SpriteID = RL062 : Player Red Left Up Stand
                    LDX        #$0010               ; Sprite Index #8 / Z-Buffer value
                    LDY        #$8086               ; Sprite X0-Y0 on screen (128,134)
                    JSR        InstallNewSprite
                    LDA        #OpponentRed1AIMove  ; Computer Player Red #1
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #OpponentRed1AIReceive
                    STA        SE_SPRITE_FUNC_RECV,X
                    JMP        ILS_End
*----
ILS_CompVsComp      LDA        #$0001               ; SpriteID = BR001 : Player Blue Right Up Stand
                    LDX        #$000E               ; Sprite Index #7 / Z-Buffer value
                    LDY        #$1086               ; Sprite X0-Y0 on screen (16,134)
                    JSR        InstallNewSprite
                    LDA        #OpponentBlueAIMove  ; Computer Player Blue #1
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #PlayerBlueAIReceive
                    STA        SE_SPRITE_FUNC_RECV,X
*-
                    LDA        #$0062               ; SpriteID = RL062 : Player Red Left Up Stand
                    LDX        #$0010               ; Sprite Index #8 / Z-Buffer value
                    LDY        #$8086               ; Sprite X0-Y0 on screen (128,134)
                    JSR        InstallNewSprite
                    LDA        #OpponentRed1AIMove  ; Computer Player Red #1
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #OpponentRed1AIReceive
                    STA        SE_SPRITE_FUNC_RECV,X
                    JMP        ILS_End
*----
ILS_PlayerVs2xComp  LDA        #$0041               ; SpriteID = RR041 : Player Red Right Up Stand
                    LDX        #$0010               ; Sprite Index #8 / Z-Buffer value
                    LDY        #$1086               ; Sprite X0-Y0 on screen (16,134)
                    JSR        InstallNewSprite
                    LDA        #OpponentRed1AIMove  ; Computer Player Red #1
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #OpponentRed1AIReceive
                    STA        SE_SPRITE_FUNC_RECV,X
*-
                    LDA        #$0001               ; SpriteID = BR001 : Player Blue Right Up Stand
                    LDX        #$000E               ; Sprite Index #7 / Z-Buffer value
                    LDY        #$4886               ; Sprite X0-Y0 on screen (72,134)
                    JSR        InstallNewSprite
                    LDA        #PlayerBlueAIMove    ; Human Player Blue #1
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #PlayerBlueAIReceive
                    STA        SE_SPRITE_FUNC_RECV,X
*-
                    LDA        #$0062               ; SpriteID = RL062 : Player Red Left Up Stand
                    LDX        #$0012               ; Sprite Index #9 / Z-Buffer value
                    LDY        #$8086               ; Sprite X0-Y0 on screen (128,134)
                    JSR        InstallNewSprite
                    LDA        #OpponentRed2AIMove  ; Computer Player Red #2
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #OpponentRed2AIReceive
                    STA        SE_SPRITE_FUNC_RECV,X                    
*---
ILS_End             RTS

*----------   InstallLevelSprite Variables   ---------  
ILS_Lvl4_Tab        HEX        880E,8937,8A5F,8C87  ; Sprite X0-Y0 on screen
ILS_Lvl8_Tab        HEX        880E,8923,8A37,8C4B
                    HEX        8F5F,9473,9C87


******************************************************************************************
*  GetGameType() :  Get Game Type 1=OneVsComp, 2=OneVsOne 0=CompVsComp (OUT:A=GameType). *
******************************************************************************************
GetGameType         STZ        GGT_Players
                    LDAL       SS_PLAYER_1          ; 1:Joystick, 2:Joypad ,3:Keyboard, 4:Computer
                    CMP        #$0004
                    BEQ        GetGameType_1
                    INC        GGT_Players
GetGameType_1       LDAL       SS_PLAYER_2          ; 1:Joystick, 2:Joypad ,3:Keyboard, 4:Computer
                    CMP        #$0004
                    BEQ        GetGameType_End
                    INC        GGT_Players
GetGameType_End     LDA        GGT_Players     
                    RTS

*----------   PlayGame Variables   ---------
GGT_Players         HEX        0000                 ; Number Human players (0-2)


****************************************************************
*  GetGameOpponent() :  Get Opponent Number (OUT:A=#Opponent). *
****************************************************************
GetGameOpponent     LDA        PG_GameType
                    BEQ        GGO_1                ; CompVsComp
                    CMP        #$0002
                    BEQ        GGO_1                ; PlayerVsPlayer
                    LDA        PG_Level
                    CMP        #$0004               ; Level 4 = Vase
                    BEQ        GGO_0
                    CMP        #$0008               ; Level 8 = Vase
                    BEQ        GGO_0
                    CMP        #$0005               ; Levels >=5
                    BGE        GGO_2
                    BRA        GGO_1                ; Levels 1,2,3
*--                    
GGO_0               LDA        #$0000               ; Breaking Vase     
                    RTS
GGO_1               LDA        #$0001               ; One Opponent     
                    RTS                    
GGO_2               LDA        #$0002               ; Two Opponents     
                    RTS


*****************************************************************************
*  GetGameIndex() :  Select Index Number 1-6 (IN:A=Level,OUT:A=Background). *
*****************************************************************************
GetGameIndex        INC                             ; Level # : 1-99
                    LSR
GGameI_Loop         CMP        #$0007
                    BLT        GGameI_End
                    SEC
                    SBC        #$0006
                    BRA        GGameI_Loop
GGameI_End          RTS                             ; Index # : 1-6


***************************************************************************************
*  LoadGameBackground() :  Load Background file data (IN:A=BgID,OUT:CarrySetOnError). *
***************************************************************************************
LoadGameBackground  DEC                             ; File already on memory ?
                    ASL
                    TAX
                    LDA        LGB_LZ4Status,X
                    BNE        LGB_CopyData
*----
LGB_LoadFile        PHX                             ; Load Background File 
                    LDA        LGB_TabFilePath,X
                    TAX                             ; File Path
                    LDA        LoadUnpackBank
                    JSL        LoadFile
                    PLX
                    BCC        LGB_StoreData
                    LDA        LGB_TabFilePath,X    ; Error
                    BRA        LGB_Error     
*--                  
LGB_StoreData       INC        LGB_LZ4Status,X      ; Store LZ4 data to Data1Bank
                    LDA        Data1Bank
                    XBA
                    ORA        LoadUnpackBank
                    STA        LGB_StoreData_1+1
                    LDA        LGB_LZ4Offset,X      ; Offset Dst
                    TAY
                    LDA        LGB_LZ4Size,X        ; Size-1
                    DEC
                    LDX        #$0000               ; Offset Src
LGB_StoreData_1     MVN        $AA,$BB
                    PHK
                    PLB
                    BRA        LGB_OK             
*----
LGB_CopyData        LDA        LoadUnpackBank       ; Copy LZ4 data to LoadUnpackBank
                    XBA
                    ORA        Data1Bank        
                    STA        LGB_CopyData_1+1
                    TXY
                    LDA        LGB_LZ4Offset,Y      ; Offset Src
                    TAX
                    LDA        LGB_LZ4Size,Y        ; Size-1
                    DEC
                    LDY        #$0000               ; Offset Dst
LGB_CopyData_1      MVN        $AA,$BB
                    PHK
                    PLB
*----
LGB_OK              CLC                             ; OK
                    RTS
*--
LGB_Error           SEC                             ; Error, Can't load file
                    RTS

*----------   LoadGameBackground Variables   ---------
LGB_LZ4Offset       HEX        0000,002E,0054,007A  ; Offset of LZ4 Backgound in Data1Bank
                    HEX        009A,00B5
LGB_LZ4Size         HEX        0C2D,2325,D025,681F  ; Size of LZ4 Backgound in Data1Bank
                    HEX        C61A,E227
LGB_LZ4Status       HEX        0000,0000,0000,0000  ; Status of LZ4 Backgound in Data1Bank (0=NotLoaded,1=Loaded)
                    HEX        0000,0000
LGB_TabFilePath     DA         LGB_FilePath001,LGB_FilePath002,LGB_FilePath003
                    DA         LGB_FilePath004,LGB_FilePath005,LGB_FilePath006
LGB_FilePath001     STRL       '1/Data/Background1.bin'
LGB_FilePath002     STRL       '1/Data/Background2.bin'
LGB_FilePath003     STRL       '1/Data/Background3.bin'
LGB_FilePath004     STRL       '1/Data/Background4.bin'
LGB_FilePath005     STRL       '1/Data/Background5.bin'
LGB_FilePath006     STRL       '1/Data/Background6.bin'


********************************************************************************
*  LoadGameMusic() :  Load Music file data (IN:A=MusicID,OUT:CarrySetOnError). *
********************************************************************************
LoadGameMusic       STA        LGM_LoadFile_ID+1
                    CMP        LGM_Current          ; File already on memory ?
                    BEQ        LGM_OK
*----
LGM_LoadFile        DEC
                    ASL
                    TAX
                    PHX                             ; Load Music File 
                    LDA        LGM_TabFilePath,X
                    TAX                             ; File Path
                    LDA        Data2Bank
                    JSL        LoadFile
                    PLX
                    BCS        LGM_Error
LGM_LoadFile_ID     LDA        #$AAAA               ; Keep current File ID
                    STA        LGM_Current     
*----
LGM_OK              DEC
                    ASL
                    TAX
                    LDA        LGM_FileTotalChunk,X ; Total number of Chunk in Music File (<= 32) 
                    STA        GMF_FileTotalChunk 
                    LDA        LGM_FileChunkSize,X  ; Size of Chunk (<=2042) 
                    STA        SGM_FileChunkSize        
                    LDA        LGM_FileTotalUnit,X  ; Total number of Unit into a Chunk (usually 8) 
                    STA        GMF_FileTotalUnit
                    LDA        LGM_FileUnitSize,X   ; Size of Unit (usually 200-256 bytes) 
                    STA        GMF_FileUnitSize
                    LDA        LGM_FileLstUnitSize,X ; Size of last Unit of the last Chunk (0-256 bytes) 
                    STA        GMF_FileLstUnitSize
                    CLC                             ; OK
                    RTS
*--
LGM_Error           LDA        LGM_TabFilePath,X    ; Error, Can't load file
                    SEC                             
                    RTS

*----------   LoadGameMusic Variables   ---------
LGM_Current         HEX        0000                 ; Current Music loaded
LGM_TabFilePath     DA         LGM_FilePath001,LGM_FilePath002,LGM_FilePath003
                    DA         LGM_FilePath004,LGM_FilePath005,LGM_FilePath006
LGM_FilePath001     STRL       '1/Data/GameMusic1.bin'
LGM_FilePath002     STRL       '1/Data/GameMusic2.bin'
LGM_FilePath003     STRL       '1/Data/GameMusic3.bin'
LGM_FilePath004     STRL       '1/Data/GameMusic4.bin'
LGM_FilePath005     STRL       '1/Data/GameMusic5.bin'
LGM_FilePath006     STRL       '1/Data/GameMusic6.bin'
LGM_FileTotalChunk  HEX        1E00,1800,1C00,1E00  ; Total number of Chunk in Music File (<= 32) 
                    HEX        1E00,0A00
LGM_FileChunkSize   HEX        D007,E007,C007,D007  ; Size of Chunk (<=2042) 
                    HEX        D007,4007
LGM_FileTotalUnit   HEX        0800,0800,0800,0800  ; Total number of Unit into a Chunk (usually 8) 
                    HEX        0800,0800
LGM_FileUnitSize    HEX        FA00,FC00,F800,FA00  ; Size of Unit (usually 230-256 bytes) 
                    HEX        FA00,E800                    
LGM_FileLstUnitSize HEX        E400,7C00,B800,F800  ; Size of last Unit of the last Chunk (0-256 bytes) 
                    HEX        DA00,7000


*******************************************************************
*  LoadGameSound() :  Load Sound file data (OUT:CarrySetOnError). *
*******************************************************************
LoadGameSound       ENT
                    PHB
                    PHK
                    PLB
                    LDX        #LGS_FilePath        ; File Path
                    LDA        Data2Bank
                    JSL        LoadFile             ; Load Sound File
                    BCS        LGS_Error
*----
                    SEI
                    LDX        #$0000               ; Sound Data Src Address in Data2Bank
                    LDY        #$E000               ; Sound Size (56 KB)
                    LDA        #$2000               ; Sound Data Dst Address in DOC Ram ($2000)
                    CLC                             ; Do NOT add 6 extra 00 at the end of the Sound
                    JSL        CopyData2BankToDOC   ; Copy Sound Data from Data2Bank to DOC Ram
                    CLI                    
*----
LGS_OK              CLC                             ; OK
                    BRA        LGC_End
*--
LGS_Error           LDA        LGS_FilePath         ; Error, Can't load file
                    SEC
*--
LGC_End             PLB                             
                    RTL

*----------   LoadGameSound Variables   ---------
LGS_FilePath        STRL       '1/Data/GameSound.bin'


***************************************************************************************
*  PlayGameSound() :  Play one Sound (IN:A=SoundID,IN:Y=Oscillator,OUT:A=SoundFrame). *
***************************************************************************************
PlayGameSound       DEC                             ; Sound ID (1-N)
                    ASL
                    TAX                           
                    LDA        PGS_SoundLengthVBL,X ; # Frames
                    STA        PGS_Frame+1
                    LDA        PGS_SoundFreqDOC,X
                    XBA
                    ORA        PGS_SoundSizeDOC,X
                    XBA
                    STA        PGS_1+1
                    LDA        PGS_SoundAddressDOC,X
                    TAX                             ; @DocRAM
PGS_1               LDA        #$AAAA               ; Size|Freq
                    JSL        PlayOneShotSound
PGS_Frame           LDA        #$AAAA               ; #Frame     
                    RTS

*----------   PlayGameSound Variables   ---------
PGS_SoundFreqDOC    HEX        D600,D600,D600,D600,D600,D600,D600,D600,D600,D600,D600 ; D6=11025 Hz
PGS_SoundSizeDOC    HEX        2D00,2D00,2D00,2400,2400,2400,2400,2400,2400,2400,2400 ; 2D=8KB, 24=4KB
PGS_SoundAddressDOC HEX        0020,0040,0060,0080,0090,00A0,00B0,00C0,00D0,00E0,00F0 ; $2000 -> $F000
PGS_SoundLengthVBL  HEX        2600,3200,2600,1900,1900,1900,1900,1900,1900,1900,1900 ; 8KB=50frames, 6KB=38frames, 4KB=25frames


******************************************
*  StartGameMusic() :  Start Game Music. *
******************************************
StartGameMusic      SEI
                    LDX        #$0000               ; Sound Data Src Address in Data2Bank
                    LDY        SGM_FileChunkSize    ; Sound Size (around 2 KB)
                    LDA        #$0000               ; Sound Data Dst Address in DOC Ram ($0000)
                    SEC                             ; Add 6 extra 00 at the end of the Sound
                    JSL        CopyData2BankToDOC   ; Copy Sound Data from Data2Bank to DOC Ram
                    CLI
*--
                    LDA        #$0001               ; Oscillators are currently playing Chuck #1-N
                    STA        GMF_ChunkOnAir
                    STZ        GMF_UnitCopied       ; Number of Unit already copied for next Chunk
                    STZ        GMF_ChunkSizeCopied  ; Size of Sound data already copied for next Chunk
                    LDA        #$0800
                    STA        GMI_DOCBaseAddress   ; DOC base Address for next Chunk Sound Data (DOC is playing 0000-07FF sound data)
                    LDA        SGM_FileChunkSize
                    STA        GMF_SrcDataAddress   ; Source Sound data Address for next Chunk Sound Data copy
*--
                    LDX        #GameMusicInt        ; Install local Sound Interrupt Function
                    LDY        #>GameMusicInt
                    LDA        #$081B               ; Sound Address for Odd Oscillator ($08=$0800) | Sound Size ($1B=2 KB)
                    JSL        PlayLoopSound        ; Play Sound located at $0000
*--
                    LDY        #$0000               ; Music Volume
SGM_Music_Loop      PHY                    
                    LDA        #$0088               ; Half Max Level
                    JSL        SetOscVolume         ; Change Music Volume
                    PLY
                    INY
                    CPY        #$0004
                    BNE        SGM_Music_Loop
*--
                    LDY        #$0004               ; Sound Volume
SGM_Sound_Loop      PHY                    
                    LDA        #$00FF               ; Max Level
                    JSL        SetOscVolume         ; Change Music Volume
                    PLY
                    INY
                    CPY        #$0020
                    BNE        SGM_Sound_Loop
*--
                    RTS

*----------   StartGameMusic Variables   ---------
SGM_FileChunkSize   HEX        0000                 ; Size of Chunk (<=2042)
                  
                  
************************************************
*  GameMusicInt() :  Interrupt Music Function. *
************************************************ 
GameMusicInt        PHB                             ; Called during the oscillators 0/1 switch 
                    PHK
                    PLB
*--
                    LDA        GMF_ChunkOnAir       ; Current Chunk # being played by the oscillators [1-N]
                    CMP        GMF_FileTotalChunk
                    BNE        GMI_NextChunk
                    LDA        #$0000               ; Reset Chunk # to 1
GMI_NextChunk       INC
                    STA        GMF_ChunkOnAir
*--
                    LDA        GMI_DOCBaseAddress   ; DOC Ram target buffer
                    BEQ        GMI_DOCBase
                    STZ        GMI_DOCBaseAddress   ; Write into DOC Ram at $0000
                    BRA        GMI_ResetCopied
GMI_DOCBase         LDA        #$0800               ; Write into DOC Ram at $0800
                    STA        GMI_DOCBaseAddress                
*--
GMI_ResetCopied     STZ        GMF_UnitCopied       ; Reset Number of Unit already copied for this Chunk
                    STZ        GMF_ChunkSizeCopied  ; Reset Size of Sound data already copied for this Chunk
*--
GMI_End             PLB
                    RTL
                    
*----------   GameMusicInt Variables   ---------
GMI_DOCBaseAddress  HEX        0000                 ; 0000:Playing [$0800-$0FFF], 0800:Playing[$0000-$07FF] 
                    

**************************************************************
*  GameMusicFeed() :  Copy chunk of Music data into DOC Ram. *
************************************************************** 
GameMusicFeed       SEI
                    LDA        GMF_ChunkOnAir       ; We are looping the sound, so reset all counters
                    CMP        GMF_FileTotalChunk
                    BNE        GMF_CopyOneUnit
*----
GMF_LoopSound       LDA        GMF_UnitCopied       ; Loop Song : Start Copy Sound from beginning
                    BNE        GMF_CopyOneUnit
                    STZ        GMF_SrcDataAddress
*----
GMF_CopyOneUnit     LDA        GMF_UnitCopied       ; Copy one Unit of Music Data every frame
                    CMP        GMF_FileTotalUnit
                    BEQ        GMF_End              ; The full Chunk have aready been copied, wait until next switch
GMF_COU_1           INC
                    CMP        GMF_FileTotalUnit
                    BEQ        GMF_COU_LastUnit     ; Copy the last Unit of the Chuck => Add Extra 00
*--
                    LDA        GMF_FileUnitSize     ; Unit Size = Normal Size
                    STA        GMF_FileCurUnitSize
                    LDA        GMI_DOCBaseAddress   ; DOC Ram base Address 0000 or 0800
                    CLC
                    ADC        GMF_ChunkSizeCopied  ; + already copied = Destination Sound Data Address in DOC 
                    STA        GMF_DstDataAddress
                    LDX        GMF_SrcDataAddress   ; Source Sound Data Address in Data2Bank
                    LDY        GMF_FileCurUnitSize  ; Sound Size = Unit Size (around 256 bytes)
                    CLC                             ; Do not add extra 00 at the end
                    JSL        CopyData2BankToDOC   ; Copy Sound Data
                    BRA        GMF_COU_Update
*--
GMF_COU_LastUnit    LDA        GMF_FileTotalChunk   ; The last Unit of the Last Chunk has a dedicated size
                    DEC
                    CMP        GMF_ChunkOnAir       
                    BNE        GMF_COU_LastUnit_1
                    LDA        GMF_FileLstUnitSize  ; Last Unit Size
                    BRA        GMF_COU_LastUnit_2
GMF_COU_LastUnit_1  LDA        GMF_FileUnitSize     ; Normal Unit size
GMF_COU_LastUnit_2  STA        GMF_FileCurUnitSize
                    LDA        GMI_DOCBaseAddress   ; DOC Ram base Address 0000 or 0800
                    CLC
                    ADC        GMF_ChunkSizeCopied  ; + already copied = Destination Sound Data Address in DOC 
                    STA        GMF_DstDataAddress
                    LDX        GMF_SrcDataAddress   ; Source Sound Data Address in Data2Bank
                    LDY        GMF_FileCurUnitSize  ; Sound Size = Unit Size (around 256 bytes)
                    SEC                             ; Add extra 00 at the end
                    JSL        CopyData2BankToDOC   ; Copy Sound Data 
*----                    
GMF_COU_Update      INC        GMF_UnitCopied       ; Another Unit copied into DOC Ram
                    LDA        GMF_SrcDataAddress   ; Next Source Data2Bank Sound data address
                    CLC
                    ADC        GMF_FileCurUnitSize
                    STA        GMF_SrcDataAddress
                    LDA        GMF_ChunkSizeCopied  ; Next Destination DOC Sound data address
                    CLC
                    ADC        GMF_FileCurUnitSize
                    STA        GMF_ChunkSizeCopied
*----
GMF_End             CLI
                    RTS

*----------   GameMusicFeed Variables   ---------
GMF_FileTotalChunk  HEX        0000                 ; Total number of Chunk in Music File (<= 32) 
GMF_FileTotalUnit   HEX        0000                 ; Total number of Unit into a Chunk (usually 8) 
GMF_FileUnitSize    HEX        0000                 ; Size of Normal Unit (usually 200-256 bytes) 
GMF_FileLstUnitSize HEX        0000                 ; Size of last Unit of the last Chunk (0-256 bytes) 
GMF_FileCurUnitSize HEX        0000                 ; Current Size of Unit (Normal or Last) 
GMF_ChunkOnAir      HEX        0000                 ; Current Chunk being played by the oscillators [1-N]
GMF_UnitCopied      HEX        0000                 ; Number of Unit already copied into DOC Ram [0-8]
GMF_SrcDataAddress  HEX        0000                 ; Source Sound Data Address into Data2Bank [0000-FFFF]
GMF_DstDataAddress  HEX        0000                 ; Destination Sound Data Address into DOC Ram [0000-FFFF]
GMF_ChunkSizeCopied HEX        0000                 ; Size of Sound Data from current Chunk alredy copied into DOC ram      

                 
****************************************
*  StopGameMusic() :  Stop Game Music. *
****************************************
StopGameMusic       JSL        FadeOutSound         ; Decrease Sound Volume
                    JSL        StopSound            ; Stop Music Oscillators (0-3)
                    RTS


*********************************************************************************************************
*  GetSequenceSoundID() :  Search for a Sound ID matching the Sequence (IN:A=SequenceID,OUT:A=SoundID). *
*********************************************************************************************************
GetSequenceSoundID  STA        GSSID_SequenceID+1
                    LDY        #$0000
GSSID_Loop          LDA        GSSID_TabSequenceID,Y
                    BEQ        GSSID_End            ; Not Found
GSSID_SequenceID    CMP        #$AAAA               ; Sequence ID
                    BNE        GSSID_Next
                    LDA        GSSID_TabSoundID,Y
                    CMP        #$FFFF
                    BNE        GSSID_End
GSSID_RandomID      LDA        #SE_SPRITE_RANDOM    ; Random Sound for Kick / Punch
                    JSL        GetRandomNumber      ; [1-100]
                    AND        #$0007               ; [0-7]
                    CMP        #$0006
                    BLT        GSSID_RandomSound
                    LSR       
GSSID_RandomSound   ASL
                    TAY
                    LDA        GSSID_TabSoundKP,Y
                    BRA        GSSID_End
GSSID_Next          INY
                    INY        
                    BRA        GSSID_Loop
GSSID_End           RTS

*----------   GetSequenceSoundID Variables   ---------
GSSID_TabSequenceID HEX        6B00,6C00,6D00,4E00,4F00,5000,3100,3200,3300,1400,1500,1600 ; Flight Kick
                    HEX        1300,3000,4D00,6A00  ; Back High Kick
                    HEX        1A00,1B00,1C00,3700,3800,3900,5400,5500,5600,7100,7200,7300 ; Touched
                    HEX        1D00,3A00,5700,7400,7500,7600,7700,7800 ; Dying
                    HEX        0E00,0F00,1000,1100,1200,1700,1800,1900 ; Other Kick + Punch
                    HEX        2B00,2C00,2D00,2E00,2F00,3400,3500,3600
                    HEX        4800,4900,4A00,4B00,4C00,5100,5200,5300
                    HEX        6500,6600,6700,6800,6900,6E00,6F00,7000
                    HEX        0000
GSSID_TabSoundID    HEX        0200,0200,0200,0200,0200,0200,0200,0200,0200,0200,0200,0200
                    HEX        0300,0300,0300,0300
                    HEX        0800,0800,0800,0800,0800,0800,0800,0800,0800,0800,0800,0800
                    HEX        0700,0700,0700,0700,0700,0700,0700,0700
                    HEX        FFFF,FFFF,FFFF,FFFF,FFFF,FFFF,FFFF,FFFF ; Random catch
                    HEX        FFFF,FFFF,FFFF,FFFF,FFFF,FFFF,FFFF,FFFF
                    HEX        FFFF,FFFF,FFFF,FFFF,FFFF,FFFF,FFFF,FFFF
                    HEX        FFFF,FFFF,FFFF,FFFF,FFFF,FFFF,FFFF,FFFF
GSSID_TabSoundKP    HEX        0400,0500,0600,0800,0A00,0B00 ; Kick / Punch

                    HEX        0100 ; Break Vase
                    HEX        0900 ; Touched
                    

******************************************************************
*  IncrementScore() :  Increment Blue Player Score (IN:A=Value). *
******************************************************************
IncrementScore      TAX                             ; Cut into 0-9 units
                    AND        #$000F
                    STA        IS_Score+6
                    TXA
                    AND        #$00F0
                    LSR
                    LSR
                    LSR
                    LSR
                    STA        IS_Score+4
                    TXA
                    XBA
                    AND        #$000F
                    STA        IS_Score+2
                    TXA
                    XBA
                    AND        #$00F0
                    LSR
                    LSR
                    LSR
                    LSR
                    STA        IS_Score+0
*--
IS_Previous         LDA        PG_Score+0           ; Backup Score
                    STA        IS_Previous_Score+0
                    LDA        PG_Score+2
                    STA        IS_Previous_Score+2
                    LDA        PG_Score+4
                    STA        IS_Previous_Score+4
                    LDA        PG_Score+6
                    STA        IS_Previous_Score+6
*--
IS_Unit             LDA        PG_Score+6           ; Update Score
                    CLC
                    ADC        IS_Score+6
                    CMP        #$000A
                    BLT        IS_Unit_1
                    INC        PG_Score+4
                    SEC
                    SBC        #$000A
IS_Unit_1           STA        PG_Score+6
*--
IS_Dozen            LDA        PG_Score+4
                    CLC
                    ADC        IS_Score+4
                    CMP        #$000A
                    BLT        IS_Dozen_1
                    INC        PG_Score+2
                    SEC
                    SBC        #$000A
IS_Dozen_1          STA        PG_Score+4
*--
IS_Hundred          LDA        PG_Score+2
                    CLC
                    ADC        IS_Score+2
                    CMP        #$000A
                    BLT        IS_Hundred_1
                    INC        PG_Score+0
                    SEC
                    SBC        #$000A
IS_Hundred_1        STA        PG_Score+2
*--
IS_Thousand         LDA        PG_Score+0
                    CLC
                    ADC        IS_Score+0
                    CMP        #$000A
                    BLT        IS_Thousand_1
                    SEC
                    SBC        #$000A
IS_Thousand_1       STA        PG_Score+0
*--
                    INC        SE_UPDATE_SCORE      ; The score need to be updated on screen
*--
                    LDA        IS_Previous_Score+2  ; New Life for the Blue Player ?
                    CMP        PG_Score+2
                    BEQ        IS_End
                    INC        PG_Lives             ; Increment Life #
                    LDA        PG_Lives
                    JSR        DrawLives            ; Update Life Counter on Screen
*--
IS_End              RTS

*----------   IncrementScore Variables   ---------
IS_Previous_Score   HEX        0000,0000,0000,0000  ; [0-9]
IS_Score            HEX        0000,0000,0000,0000  ; [0-9]


**********************************************************
*  DrawScore() :  Display Score on Screen (IN:A=@Score). *
**********************************************************
DrawScore           STA        DS_1+1               ; Init
                    INC
                    INC
                    STA        DS_2+1
                    INC
                    INC
                    STA        DS_3+1
                    INC
                    INC
                    STA        DS_4+1
*--
DS_1                LDA        $AAAA
                    ASL
                    TAX
                    LDAL       Score_Bitmap_Tab,X
                    TAX                             ; @Src
                    LDY        #$2848               ; @Dst 80,13
                    LDA        #$0208               ; Width4|Heigh
                    JSL        CopyOneBitmap
*--
DS_2                LDA        $AAAA
                    ASL
                    TAX
                    LDAL       Score_Bitmap_Tab,X
                    TAX                             ; @Src
                    LDY        #$284C               ; @Dst 88,13
                    LDA        #$0208               ; Width4|Heigh
                    JSL        CopyOneBitmap
*--
DS_3                LDA        $AAAA
                    ASL
                    TAX
                    LDAL       Score_Bitmap_Tab,X
                    TAX                             ; @Src
                    LDY        #$2850               ; @Dst 96,13
                    LDA        #$0208               ; Width4|Heigh
                    JSL        CopyOneBitmap
*--
DS_4                LDA        $AAAA
                    ASL
                    TAX
                    LDAL       Score_Bitmap_Tab,X
                    TAX                             ; @Src
                    LDY        #$2854               ; @Dst 104,13
                    LDA        #$0208               ; Width4|Heigh
                    JSL        CopyOneBitmap
*--
                    LDAL       #Score_0             ; @Src
                    TAX
                    LDY        #$2858               ; @Dst 112,13
                    LDA        #$0208               ; Width4|Heigh
                    JSL        CopyOneBitmap
*--
                    LDAL       #Score_0             ; @Src
                    TAX
                    LDY        #$285C               ; @Dst 120,13
                    LDA        #$0208               ; Width4|Heigh
                    JSL        CopyOneBitmap
*--
                    RTS


**********************************************************
*  DrawLives() :  Display #Lives on Screen (IN:A=Lives). *
**********************************************************
DrawLives           CMP        #$000A               ; 1 Digit only
                    BLT        DrawLives_1
                    LDA        #$0009
DrawLives_1         ASL                             ; Display 
                    TAX
                    LDAL       Score_Bitmap_Tab,X
                    TAX                             ; @Src
                    LDY        #$2884               ; @Dst 200,13
                    LDA        #$0208               ; Width4|Heigh
                    JSL        CopyOneBitmap
                    RTS


*********************************************************
*  DrawLevel() :  Display Level on Screen (IN:A=Level). *
*********************************************************
DrawLevel           STZ        DL_Dozen             ; Init
*--
DL_CheckDozen       CMP        #$000A               ; Hexa -> Dec
                    BLT        DL_StoreUnit
                    SEC
                    SBC        #$000A
                    INC        DL_Dozen
                    BRA        DL_CheckDozen
DL_StoreUnit        STA        DL_Unit
*--
                    LDA        DL_Dozen
                    ASL
                    TAX
                    LDAL       Score_Bitmap_Tab,X
                    TAX                             ; @Src
                    LDY        #$28AD               ; @Dst 282,13
                    LDA        #$0208               ; Width4|Heigh
                    JSL        CopyOneBitmap
*--
                    LDA        DL_Unit
                    ASL
                    TAX
                    LDAL       Score_Bitmap_Tab,X
                    TAX                             ; @Src
                    LDY        #$28B1               ; @Dst 290,13
                    LDA        #$0208               ; Width4|Heigh
                    JSL        CopyOneBitmap
*--
                    RTS
                    
*----------   DrawLevel Variables   ---------
DL_Dozen            HEX        0000
DL_Unit             HEX        0000


******************************************************************************
*  DecrementLifeBar() :  Decrement Life Bar (A:IN=DecrementValue,X:=Player). *
******************************************************************************
DecrementLifeBar    CMP        #$0000               ; Nothing to decrement
                    BNE        DLB_Target
                    LDA        #$0001               ; Minimal Damage = 1
*--
DLB_Target          CPX        #$000E               ; Index #7 = Blue
                    BEQ        DLB_Blue
                    CPX        #$0010               ; Index #8 = Red #1
                    BEQ        DLB_Red1
                    CPX        #$0012               ; Index #9 = Red #2
                    BEQ        DLB_Red2
                    JMP        DLB_End
*--
DLB_Blue            CMP        PL_LifeBar_Blue      ; Player Blue
                    BLT        DLB_Blue_1
                    LDA        PL_LifeBar_Blue
DLB_Blue_1          STA        DLB_DecrementValue
                    LDA        PL_LifeBar_Blue      ; Already 0
                    STA        DLB_CurrentValue
                    BEQ        DLB_Blue_End
                    SEC
                    SBC        DLB_DecrementValue
                    STA        PL_LifeBar_Blue      ; Update Blue Life
                    BNE        DLB_Blue_2
                    JSR        GetPLayerBlueIndex   ; Player Blue just died
                    LDA        #$FFFF
                    STA        SE_SPRITE_NEXT_SEQ,X ; Next Sequence is Death
DLB_Blue_2          LDX        #$2033
                    BRA        DLB_EraseScreen
DLB_Blue_End        JMP        DLB_End
*--
DLB_Red1            CMP        PL_LifeBar_Red1      ; Player Red #1
                    BLT        DLB_Red1_1
                    LDA        PL_LifeBar_Red1
DLB_Red1_1          STA        DLB_DecrementValue
                    LDA        PL_LifeBar_Red1      ; Already 0
                    BEQ        DLB_Red1_End
                    STA        DLB_CurrentValue
                    SEC
                    SBC        DLB_DecrementValue
                    STA        PL_LifeBar_Red1      ; Update Red #1 Life
                    BNE        DLB_Red1_2
                    JSR        GetPLayerRed1Index   ; Player Red #1 just died
                    LDA        #$FFFF
                    STA        SE_SPRITE_NEXT_SEQ,X ; Next Sequence is Death
DLB_Red1_2          LDX        #$22B3
                    BRA        DLB_EraseScreen
DLB_Red1_End        JMP        DLB_End          
*--
DLB_Red2            CMP        PL_LifeBar_Red2      ; Player Red #2
                    BLT        DLB_Red2_1
                    LDA        PL_LifeBar_Red2
DLB_Red2_1          STA        DLB_DecrementValue
                    LDA        PL_LifeBar_Red2      ; Already 0
                    BEQ        DLB_Red2_End
                    STA        DLB_CurrentValue
                    SEC
                    SBC        DLB_DecrementValue
                    STA        PL_LifeBar_Red2      ; Update Red #2 Life
                    BNE        DLB_Red2_2
                    JSR        GetPLayerRed2Index   ; Player Red #2 just died
                    LDA        #$FFFF
                    STA        SE_SPRITE_NEXT_SEQ,X ; Next Sequence is Death
DLB_Red2_2          LDX        #$2533
                    BRA        DLB_EraseScreen
DLB_Red2_End        JMP        DLB_End
*---
DLB_EraseScreen     STX        DLB_ScreenOffset
                    LDA        DLB_CurrentValue     ; Delete last Odd Pixel
                    DEC
                    LSR
                    BCS        DLB_EraseScreen_2
                    CLC
                    ADC        DLB_ScreenOffset
                    TAX
                    LDA        #$0000
                    STAL       $010000,X
                    STAL       $0100A0,X
                    STAL       $010140,X
                    DEC        DLB_CurrentValue
                    DEC        DLB_DecrementValue    
*--                    
DLB_EraseScreen_2   LDA        DLB_DecrementValue   ; Delete two pixels set
                    LSR
                    BEQ        DLB_EraseScreen_5
                    STA        DLB_EraseScreen_4+1
                    LDY        #$0000
DLB_EraseScreen_3   LDA        DLB_CurrentValue
                    LSR
                    DEC
                    CLC
                    ADC        DLB_ScreenOffset
                    TAX
                    LDA        #$0000
                    STAL       $010000,X
                    STAL       $0100A0,X
                    STAL       $010140,X
                    DEC        DLB_CurrentValue
                    DEC        DLB_CurrentValue
                    DEC        DLB_DecrementValue    
                    DEC        DLB_DecrementValue
                    INY
DLB_EraseScreen_4   CPY        #$AAAA
                    BNE        DLB_EraseScreen_3
*--
DLB_EraseScreen_5   LDA        DLB_DecrementValue   ; Delete last Even pixel
                    BEQ        DLB_End
                    LDA        DLB_CurrentValue
                    DEC
                    LSR
                    CLC
                    ADC        DLB_ScreenOffset
                    TAX
                    LDAL       $010000,X
                    AND        #$FFF0
                    STAL       $010000,X
                    LDAL       $0100A0,X
                    AND        #$FFF0
                    STAL       $0100A0,X
                    LDAL       $010140,X
                    AND        #$FFF0
                    STAL       $010140,X
*-- 
DLB_End             RTS

*----------   DecrementLifeBar Variables   ---------
DLB_CurrentValue    HEX        0000                 ; Current life value (0-200)
DLB_DecrementValue  HEX        0000
DLB_ScreenOffset    HEX        0000


*********************************************************
*  FillUpBlueLifeBar() :  Fill up Blue Player Life Bar. *
*********************************************************
FillUpBlueLifeBar   LDA        #$00C8               ; 200
                    STA        PL_LifeBar_Blue
*--
                    LDX        #$0062               ; Player #1 (Blue)
FUBLB_Loop          LDA        #$7777
                    STAL       $012033,X
                    STAL       $0120D3,X
                    STAL       $012173,X
                    DEX
                    DEX
                    BPL        FUBLB_Loop
*--                    
                    RTS


********************************************************
*  GetGlobalInput() :  Get global Input (OUT:A=Input). *
********************************************************
GetGlobalInput      STZ        GGI_End_1+1
                    SEP        #$30
*--
GGI_Keyboard        LDAL       KBDREAD              ; Keyboard
                    BPL        GGI_End
                    AND        #$7F
                    CMP        #$1B                 ; Esc
                    BEQ        GGI_Keyboard_End
                    CMP        #$50                 ; P
                    BEQ        GGI_Keyboard_P
                    CMP        #$70                 ; p
                    BEQ        GGI_Keyboard_P
                    CMP        #$41                 ; A
                    BEQ        GGI_Keyboard_A
                    CMP        #$61                 ; a
                    BEQ        GGI_Keyboard_A
                    BRA        GGI_End
*--
GGI_Keyboard_P      LDA        #$50                 ; P = Pause
                    BRA        GGI_Keyboard_End
GGI_Keyboard_A      LDA        #$41                 ; A = Audio
GGI_Keyboard_End    STA        GGI_End_1+1
                    STAL       KBDSTRB              ; Clear Strobe only if we have pulled the input
*--
GGI_End             REP        #$30
GGI_End_1           LDA        #$0000
                    RTS


**********************************************************************************
*  GetPlayer1Input() :  Get Input for Player 1 (IN:Y=JoystickInput,OUT:A=Input). *
**********************************************************************************
GetPlayer1Input     LDAL       SS_PLAYER_1          ; 1:Joystick, 2:Joypad, 3:Keyboard, 4:Computer
                    CMP        #$0001
                    BEQ        GP1I_Joystick
                    CMP        #$0002
                    BEQ        GP1I_Joypad
                    CMP        #$0003
                    BEQ        GP1I_Keyboard
                    LDA        #$0035
                    BRA        GP1I_End
*--
GP1I_Joystick       TYA                             ; Apple Analog Joystick  
                    BRA        GP1I_End        
*--
GP1I_Joypad         LDX        PG_JoypadSlot        ; Joypad #1 
                    JSL        GetJoypad1Input
                    BRA        GP1I_End        
*--
GP1I_Keyboard       JSL        GetKeyboardInput     ; Keyboard
                    BRA        GP1I_End
*--
GP1I_End            RTS


**********************************************************************************
*  GetPlayer2Input() :  Get Input for Player 2 (IN:Y=JoystickInput,OUT:A=Input). *
**********************************************************************************
GetPlayer2Input     LDAL       SS_PLAYER_2          ; 1:Joystick, 2:Joypad, 3:Keyboard, 4:Computer
                    CMP        #$0001
                    BEQ        GP2I_Joystick
                    CMP        #$0002
                    BEQ        GP2I_Joypad
                    CMP        #$0003
                    BEQ        GP2I_Keyboard
                    LDA        #$0035
                    BRA        GP2I_End
*--
GP2I_Joystick       TYA                             ; Apple Analog Joystick  
                    BRA        GP2I_End        
*--
GP2I_Joypad         LDAL       SS_PLAYER_1          ; Switch to Joypad #2 if Player #1 is using Joypad #1
                    CMP        #$0002               
                    BEQ        GP2I_Joypad_2
GP2I_Joypad_1       LDX        PG_JoypadSlot        ; Joypad #1 
                    JSL        GetJoypad1Input
                    BRA        GP2I_End        
GP2I_Joypad_2       LDX        PG_JoypadSlot        ; Joypad #2 
                    JSL        GetJoypad2Input
                    BRA        GP2I_End        
*--
GP2I_Keyboard       JSL        GetKeyboardInput     ; Keyboard
                    BRA        GP2I_End
*--
GP2I_End            RTS


*************************************************************
*   GetPLayerXXXXIndex() :  Get Player Index (OUT:X=Index). *
*************************************************************
GetPLayerBlueIndex  LDX        #$000E               ; #7
                    RTS
GetPLayerRed1Index  LDX        #$0010               ; #8
                    RTS
GetPLayerRed2Index  LDX        #$0012               ; #9
                    RTS


***********************************************************************
*  ProcessGameTimer() :  Process all time-based Events (IN:A=Frame#). *
***********************************************************************
ProcessGameTimer    PHX                             ; Time based Events
                    STA        PGT_Frame
*--
                    LDA        PG_Level
                    CMP        #$0003               ; Nothing before Level #3
                    BLT        PGT_LevelVaseEnd
                    CMP        #$0004               ; Level 4 : Vase
                    BEQ        PGT_LevelVase
                    CMP        #$0008               ; Level 8 : Vase
                    BEQ        PGT_LevelVase
                    LDA        PG_GameType
                    BEQ        PGT_CompVsComp       ; Computer Vs Computer
                    CMP        #$0002
                    BEQ        PGT_PlayerVsPlayer   ; Player Vs Player
                    BRA        PGT_PlayerVsComp
*--
PGT_LevelVase       JSR        GetActiveVase        ; Bouncing Vase on Screen ?
                    BEQ        PGT_LevelVaseEnd
                    LDA        PGT_Frame
                    CMP        #$0016               ; Wait 2 sec before decreasing life
                    BLT        PGT_LevelVaseEnd
                    LDA        #$0001               ; Breaking Vase Level
                    LDX        #$000E               ; Blue Player Index
                    JSR        DecrementLifeBar     ; Decrement Blue Life Bar
PGT_LevelVaseEnd    JMP        PGT_End
*--
PGT_CompVsComp      JMP        PGT_End              ; Computer Vs Computer
*--
PGT_PlayerVsPlayer  JMP        PGT_End              ; Player Vs Player
*--
PGT_PlayerVsComp    LDA        PGT_Frame            ; Frame Based Event
                    CMP        #$00FA         
                    BEQ        PGT_BonusVase        ; Bonus Vase at 20 sec 
*-
PGT_PVC_RS          CMP        #$0064               ; Right Shuriken : 8 sec (64) + Every 40 seconds +=1F4
                    BLT        PGT_PVC_End
                    SEC                   
                    SBC        #$0064
                    BEQ        PGT_ShurikenRight
PGT_PVC_RS_Loop     CMP        #$01F4
                    BLT        PGT_PVC_LS
                    SEC
                    SBC        #$01F4
                    BEQ        PGT_ShurikenRight
                    BRA        PGT_PVC_RS_Loop
*-
PGT_PVC_LS          LDA        PGT_Frame            ; Left Shuriken : 28 sec (15E) + Every 40 seconds +=1F4
                    CMP        #$015E
                    BLT        PGT_PVC_End
                    SEC                   
                    SBC        #$015E
                    BEQ        PGT_ShurikenLeft                    
PGT_PVC_LS_Loop     CMP        #$01F4   
                    BLT        PGT_PVC_End
                    SEC
                    SBC        #$01F4
                    BEQ        PGT_ShurikenLeft 
                    BRA        PGT_PVC_LS_Loop                    
*-
PGT_PVC_End         BRA        PGT_End
*--
PGT_ShurikenRight   LDA        #$0089               ; Add a new Shuriken coming from Right / Sprite ID = 0089 / 008A
                    LDX        #$0002               ; Sprite Index #1 x2 / Z-Buffer value
                    LDY        #$9DB5               ; Sprite X0|Y0 on screen (314,181)
                    JSR        InstallNewSprite
                    LDA        #ShurikenAIMove      ; Shuriken AI Move Function
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #ShurikenAISend      ; Shuriken AI Send Function
                    STA        SE_SPRITE_FUNC_SEND,X
                    LDA        #$0001               ; Shuriken goes left
                    STA        SE_SPRITE_DATA_1,X
                    BRA        PGT_End
*--
PGT_ShurikenLeft    LDA        #$0089               ; Add a new Shuriken coming from Left / Sprite ID = 0089 / 008A
                    LDX        #$0004               ; Sprite Index #2 x2 / Z-Buffer value
                    LDY        #$00B5               ; Sprite X0|Y0 on screen (0,181)
                    JSR        InstallNewSprite
                    LDA        #ShurikenAIMove      ; Shuriken AI Move Function
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #ShurikenAISend      ; Shuriken AI Send Function
                    STA        SE_SPRITE_FUNC_SEND,X
                    LDA        #$0002               ; Shuriken goes Right
                    STA        SE_SPRITE_DATA_1,X
                    BRA        PGT_End
*--
PGT_BonusVase       LDA        #SE_SPRITE_RANDOM    ; Add a new Bonus Vase with Random X0 position
                    JSL        GetRandomNumber      ; [1-100]
                    CLC
                    ADC        #$0020               ; Y0=8C, X+=32 [33-133]
                    ORA        #$8C00
                    XBA
                    STA        PGT_BonusVase_X0Y0+1
                    XBA
                    DEC
                    DEC
                    XBA
                    STA        PGT_BonusVase_Base+1
                    LDA        #$0081               ; Sprite ID = 0081
                    LDX        #$0006               ; Sprite Index x2 / Z-Buffer value
PGT_BonusVase_X0Y0  LDY        #$548C               ; Sprite X0-Y0 on screen (168/2,140)
                    JSR        InstallNewSprite
                    LDA        #BonusVaseAIMove     ; Bonus Vase AI Move Function
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #BonusVaseAIReceive  ; Bonus Vase AI Receive Function
                    STA        SE_SPRITE_FUNC_RECV,X
                    LDA        #$0000               ; Bonus Vase Sequence #
                    STA        SE_SPRITE_DATA_1,X
PGT_BonusVase_Base  LDA        #$528C               ; Bonus Vase X0Y0 base (X0 -= 2)
                    STA        SE_SPRITE_DATA_2,X                    
                    BRA        PGT_End
*--
PGT_End             PLX
                    RTS
                    
*----------   ProcessGameTimer Variables   ---------  
PGT_Frame           HEX        0000
PGT_Unit            HEX        0000


*************************************************************************
*  AddImpactSprite() :  Add a new Hit Impact Sprite (IN:A=ImpactX0|Y0). *
*************************************************************************
AddImpactSprite     TAY                             ; Sprite X0|Y0 on screen
                    STY        AIS_Install_X0YO+1
                    LDA        SE_MAX_SPRITE_NB     ; Search for a free place
                    ASL
                    STA        AIS_Limit+1
                    LDX        #$0000
AIS_Loop            LDA        SE_SPRITE_STATUS,X
                    BEQ        AIS_Install
                    INX
                    INX
AIS_Limit           CPX        #$AAAA
                    BNE        AIS_Loop
                    BRA        AIS_End              ; No Free Space
*----
AIS_Install         LDA        #$0085               ; Add a new Impact / Sprite ID = 0085 / 0088
                    JSR        InstallNewSprite
                    LDA        #ImpactAIMove        ; Impact AI Move Function
                    STA        SE_SPRITE_FUNC_MOVE,X
                    LDA        #$0000               ; Impact Sequence # (0-3)
                    STA        SE_SPRITE_DATA_1,X
AIS_Install_X0YO    LDA        #$AAAA               ; Root Sprite X0|Y0 
                    STA        SE_SPRITE_DATA_2,X
*----
AIS_End             RTS        


*******************************************************************
*  InstallPreComputed() :  Install Pre-computed data into tables. *
*******************************************************************
InstallPreComputed  LDX        #$0116               ; 280 bytes
IPC_Loop_1          LDA        SPRITE_RECEIVE_TABP,X
                    STA        SPRITE_RECEIVE_TAB,X
                    DEX
                    DEX
                    BPL        IPC_Loop_1
*--
                    LDX        #$1194               ; 4502 bytes
IPC_Loop_2          LDA        TabEraseCodeSizeP,X
                    STA        TabEraseCodeSize,X
                    DEX
                    DEX
                    BPL        IPC_Loop_2
*--
                    LDX        #$0256               ; Remap Erase Code Banks
IPC_Loop_3          LDA        TAB_ERASE_BANK,X     ; Erase ID Code Bank   
                    CMP        #$1900
                    BNE        IPC_Next_1
                    LDA        EraseCode1Bank
                    BRA        IPC_Next_3
IPC_Next_1          CMP        #$1A00
                    BNE        IPC_Next_2
                    LDA        EraseCode2Bank
                    BRA        IPC_Next_3
IPC_Next_2          CMP        #$1B00
                    BNE        IPC_Next_4
                    LDA        EraseCode3Bank
IPC_Next_3          STA        TAB_ERASE_BANK,X
IPC_Next_4          DEX
                    DEX
                    BPL        IPC_Loop_3
*--
                    LDX        #$0112               ; Remap TAB_ERASE_BEGIN Address
IPC_Loop_4          LDA        TAB_ERASE_BEGIN,X    ; Erase ID Code Bank   
                    SEC
                    SBC        #$4FD6               ; ERASE_SPR_000 address
                    CLC
                    ADC        #ERASE_SPR_000
                    STA        TAB_ERASE_BEGIN,X
                    DEX
                    DEX
                    BPL        IPC_Loop_4
*--
                    RTS
                    
                    
*************************************************************************************
*  CheckGameEnding() :  Check Game Ending status (OUT:A=ExitStatus,OUT:X=ExitCode). *
*************************************************************************************
CheckGameEnding     LDA        PG_GameType           ; GameType : 0:CompVsComp, 1:PlayerVsComp, 2:PlayerVsPlayer
                    BEQ        CGE_TwoPlayers
                    CMP        #$0002
                    BEQ        CGE_TwoPlayers
*----
CGE_OnePlayer       LDA        PL_LifeBar_Blue       ; Games stops if either Player Blue is dead or if all Red are Dead / all Vase broken
                    BEQ        CGE_OnePlayer_Lose
                    LDA        PG_Level
                    CMP        #$0004
                    BEQ        CGE_OnePlayerVase
                    CMP        #$0008
                    BEQ        CGE_OnePlayerVase
*--
CGE_OnePlayerRed    LDA        PL_LifeBar_Red1       ; Check Red Player #1
                    BNE        CGE_Continue          ; Game continues
                    LDA        PL_LifeBar_Red2       ; Check Red Player #2
                    BNE        CGE_Continue          ; Game continues
                    BRA        CGE_OnePlayer_Win
*--
CGE_OnePlayerVase   JSR        GetActiveVase         ; Check Remaining Vase #
                    BNE        CGE_Continue          ; Game continues 
*--
CGE_OnePlayer_Win   LDX        #$0000                ; Player Blue Win
                    LDA        #$0020                ; Game Stops in 32 frames
                    BRA        CGE_End
*--
CGE_OnePlayer_Lose  LDX        #$0001                ; Player Blue lose
                    LDA        #$0020                ; Game Stops in 32 frames
                    BRA        CGE_End
*----
CGE_TwoPlayers      LDA        PL_LifeBar_Blue       ; Games stops if one of the two players is dead
                    BEQ        CGE_TwoPlayers_Stop
                    LDA        PL_LifeBar_Red1
                    BEQ        CGE_TwoPlayers_Stop
                    LDA        #$0000                ; Game continues
                    BRA        CGE_End
CGE_TwoPlayers_Stop LDX        #$0001                ; Status : Lose
                    LDA        #$0020                ; Game Stops in 32 frames
                    BRA        CGE_End
*----
CGE_Continue        LDA        #$0000                ; Game Continues
CGE_End             RTS


********************************************************************************
*  GetDeathSequenceID() :  Compute Death Sequence ID (IN:X=Index,OUT:A=SeqID). *
********************************************************************************
GetDeathSequenceID  LDY        #$0000
GDSID_Loop          LDA        GDSID_SpriteID,Y
                    CMP        SE_SPRITE_ID,X
                    BEQ        GDSID_End
                    INY
                    INY
                    BRA        GDSID_Loop
GDSID_End           LDA        GDSID_SequenceID,Y
                    RTS

*----------   GetDeathSequenceID Variables   ---------
GDSID_SpriteID      HEX        0100,1A00,2200,3B00 ; Blue  Right/Left
                    HEX        4100,5A00,6200,7B00 ; Red   Right/Left
GDSID_SequenceID    HEX        1D00,7600,3A00,7500
                    HEX        5700,7800,7400,7700


********************************************************************************
*  GetActiveVase() :  Count the number of active Vase on Screen (OUT:A=#Vase). *
********************************************************************************
GetActiveVase       LDX        #$0000               ; Level 4 & 8
                    LDY        #$0000
                    STY        GAV_COUNT
GetActiveVase_1     LDA        SE_SPRITE_ID,X
                    CMP        #$0040
                    BNE        GetActiveVase_2
                    INC        GAV_COUNT
GetActiveVase_2     INX
                    INX
                    INY
                    CPY        SE_MAX_SPRITE_NB
                    BNE        GetActiveVase_1
*--
                    LDA        GAV_COUNT
                    RTS

*----------   GetActiveVase Variables   ---------
GAV_COUNT           HEX        0000
                    

**************************************************************************************************************
*  GetTouchedSequence() :  Compute the Touched Sequence ID (IN:X=RecvIndex,IN:Y=SendIndex,OUT:A=SequenceID). *
**************************************************************************************************************
GetTouchedSequence  STX        GTS_Receive_Index
                    STY        GTS_Send_Index
                    STZ        GTS_Hit_Direction
                    STZ        GTS_SpriteDirection
                    STZ        GTS_SpriteColor
*--
                    LDA        SE_SPRITE_NEXT_SEQ,X ; Check if an Extra Sequence is already selected
                    BNE        GTS_GoNoSequence           
                    LDA        SE_SPRITE_STATUS,X   ; Check if Receive Sprite is in Motion
                    CMP        #$0002               ; Pending
                    BNE        GTS_GoNoSequence
                    LDA        SE_SPRITE_ID,X       ; Check if Receive is Stand-up (not Duck)
                    JSR        IsDuckSprite
                    BCC        GTS_Touched 
GTS_GoNoSequence    JMP        GTS_NoSequence
*--
GTS_Touched         LDA        SE_SPRITE_X0,X       ; Find Hit Direction
                    CMP        SE_SPRITE_X0,Y
                    BLT        GTS_Direction        ; Hit goes Left
                    INC        GTS_Hit_Direction    ; Hit goes Right
*--
GTS_Direction       LDA        SE_SPRITE_ID,X       ; Find Receive Sprite Direction
                    JSR        GetSpriteDirection
                    BCS        GTS_Right
                    JMP        GTS_Left
*------
GTS_Right           INC        GTS_SpriteDirection  ; Receive Sprite is facing Right
                    LDA        SE_SPRITE_ID,X       
                    JSR        GetSpriteColor       ; Find Sprite Color
                    BCS        GTS_Right_Red
*--
GTS_Right_Blue      LDA        GTS_Hit_Direction    ; Check Space
                    BNE        GTS_RB_Right
*-
GTS_RB_Left         LDA        SE_SPRITE_X0,X       ; Hit goes Left
                    CMP        GTS_ScreenLeftX      ; Left Border
                    BLT        GTS_RB_Place            
                    LDA        #$001B               ; Player Blue Right : Touched go Left
                    JMP        GTS_End
*-
GTS_RB_Right        LDA        SE_SPRITE_X0,X       ; Hit goes Right
                    CMP        GTS_ScreenRightX     ; Right Border
                    BGE        GTS_RB_Place        
                    LDA        #$001C               ; Player Blue Right : Touched go Right
                    JMP        GTS_End
*-
GTS_RB_Place        LDA        #$001A               ; Player Blue Right : Touched in Place
                    BRA        GTS_End                    
*----
GTS_Right_Red       INC        GTS_SpriteColor
                    LDA        GTS_Hit_Direction    ; Check Space
                    BNE        GTS_RR_Right
*-
GTS_RR_Left         LDA        SE_SPRITE_X0,X       ; Hit goes Left
                    CMP        GTS_ScreenLeftX      ; Left Border
                    BLT        GTS_RR_Place            
                    LDA        #$0055               ; Player Red Right : Touched go Left
                    BRA        GTS_End
*-
GTS_RR_Right        LDA        SE_SPRITE_X0,X       ; Hit goes Right
                    CMP        GTS_ScreenRightX     ; Right Border
                    BGE        GTS_RR_Place        
                    LDA        #$0056               ; Player Red Right : Touched go Right
                    BRA        GTS_End
*-
GTS_RR_Place        LDA        #$0054               ; Player Red Right : Touched in Place
                    BRA        GTS_End
*------
GTS_Left            LDA        SE_SPRITE_ID,X       ; Receive Sprite is facing Right         
                    JSR        GetSpriteColor       ; Find Sprite Color  
                    BCS        GTS_Left_Red
*----
GTS_Left_Blue       LDA        GTS_Hit_Direction    ; Check Space
                    BNE        GTS_LB_Right
*-
GTS_LB_Left         LDA        SE_SPRITE_X0,X       ; Hit goes Left
                    CMP        GTS_ScreenLeftX      ; Left Border
                    BLT        GTS_LB_Place            
                    LDA        #$0038               ; Player Blue Left : Touched go Left
                    BRA        GTS_End
*-
GTS_LB_Right        LDA        SE_SPRITE_X0,X       ; Hit goes Right
                    CMP        GTS_ScreenRightX     ; Right Border
                    BGE        GTS_LB_Place        
                    LDA        #$0039               ; Player Blue Left : Touched go Right
                    BRA        GTS_End
*-
GTS_LB_Place        LDA        #$0037               ; Player Blue Left : Touched in Place
                    BRA        GTS_End                    
*----
GTS_Left_Red        INC        GTS_SpriteColor
                    LDA        GTS_Hit_Direction    ; Check Space
                    BNE        GTS_LR_Right 
*-
GTS_LR_Left         LDA        SE_SPRITE_X0,X       ; Hit goes Left
                    CMP        GTS_ScreenLeftX      ; Left Border
                    BLT        GTS_LR_Place            
                    LDA        #$0072               ; Player Red Left : Touched go Left
                    BRA        GTS_End
*-
GTS_LR_Right        LDA        SE_SPRITE_X0,X       ; Hit goes Right
                    CMP        GTS_ScreenRightX     ; Right Border
                    BGE        GTS_LR_Place        
                    LDA        #$0073               ; Player Red Left : Touched go Right
                    BRA        GTS_End
*-
GTS_LR_Place        LDA        #$0071               ; Player Red Left : Touched in Place
                    BRA        GTS_End                   
*----
GTS_NoSequence      LDA        #$0000               ; No Touched Sequence available
GTS_End             RTS

*----------   GetTouchedSequence Variables   ---------
GTS_Receive_Index   HEX        0000                 ; Receive Sprite Index
GTS_Send_Index      HEX        0000                 ; Send Sprite Index
GTS_Hit_Direction   HEX        0000                 ; 0:Left / 1:Right
GTS_SpriteDirection HEX        0000                 ; 0:Left / 1:Right
GTS_SpriteColor     HEX        0000                 ; 0:Blue / 1:Red
GTS_ScreenLeftX     HEX        0D00                 ; 26                 
GTS_ScreenRightX    HEX        8000                 ; 256


******************************************************************************************
*  GetSpriteDirection() :  Compute Sprite direction (IN:A=SpriteID,OUT:Carry=Direction). *
******************************************************************************************
GetSpriteDirection  CMP        #$0020               ; Right 001-01E
                    BLT        GSD_Right
                    CMP        #$0040               ; Left  022-03F
                    BLT        GSD_Left
                    CMP        #$005F               ; Right 041-05E
                    BLT        GSD_Right
                    CMP        #$0080               ; Left  062-07F
                    BLT        GSD_Left                                        
*--
GSD_Left            CLC                             ; Sprite goes Left
                    RTS
*--
GSD_Right           SEC                             ; Sprite goes Right
                    RTS


******************************************************************************
*  GetSpriteColor() :  Compute Sprite color (IN:A=SpriteID,OUT:Carry=Color). *
******************************************************************************
GetSpriteColor      CMP        #$0040               ; Blue 01-3F
                    BLT        GSC_Blue
*--
GSC_Red             SEC                             ; Sprite is Red
                    RTS
*--
GSC_Blue            CLC                             ; Sprite is Blue
                    RTS


*********************************************************************************
*  IsDuckSprite() :  Check for Duck position (IN:A=SpriteID,OUT:Carry=1IfDuck). *
*********************************************************************************
IsDuckSprite        PHX                             ; Init
                    STA        IDS_SpriteID+1
                    LDX        #$0000
IDS_Loop            LDA        IDS_TableSpriteID,X
                    BEQ        IDS_NoDuck
IDS_SpriteID        CMP        #$AAAA
                    BEQ        IDS_Duck
                    INX
                    INX
                    BRA        IDS_Loop
*--
IDS_NoDuck          PLX
                    CLC                             ; Sprite is NOT Duck
                    RTS
*--
IDS_Duck            PLX
                    SEC                             ; Sprite is Duck
                    RTS  
                    
*----------   IsDuckSprite Variables   ---------                    
IDS_TableSpriteID   HEX        1A00,1B00,1C00,1D00
                    HEX        3B00,3C00,3D00,3E00
                    HEX        5A00,5B00,5C00,5D00
                    HEX        7B00,7C00,7D00,7E00
                    HEX        0000                 ; End of Table


***************************************************************************************
*  IsFlightSprite() :  Check for Flight position (IN:A=SpriteID,OUT:Carry=1IfFlight). *
***************************************************************************************
IsFlightSprite      PHX                             ; Init
                    STA        IFS_SpriteID+1
                    LDX        #$0000
IFS_Loop            LDA        IFS_TableSpriteID,X
                    BEQ        IFS_NoFlight
IFS_SpriteID        CMP        #$AAAA
                    BEQ        IFS_Flight
                    INX
                    INX
                    BRA        IFS_Loop
*--
IFS_NoFlight        PLX
                    CLC                             ; Sprite is NOT Flight
                    RTS
*--
IFS_Flight          PLX
                    SEC                             ; Sprite is Flight
                    RTS  
                    
*----------   IsFlightSprite Variables   ---------                    
IFS_TableSpriteID   HEX        0F00,1000,1100,1200,1300,1400,1500,1600,1700,1800,1900
                    HEX        3000,3100,3200,3300,3400,3500,3600,3700,3800,3900,3A00
                    HEX        4F00,5000,5100,5200,5300,5400,5500,5600,5700,5800,5900
                    HEX        7000,7100,7200,7300,7400,7500,7600,7700,7800,7900,7A00
                    HEX        0000                 ; End of Table


***************************************************************************************************
*  GetOpponentLocation() :  Compute Opponent Location (IN:X=Player,IN:Y=Opponent,OUT:A=Location). * 
***************************************************************************************************
GetOpponentLocation LDA        SE_SPRITE_ID,X       ; Get Player Direction
                    JSR        GetSpriteDirection
                    BCS        GOL_Right
                    JMP        GOL_Left
*----
GOL_Right           JSR        IsAboutSamePosition  ; Player goes Right
                    BNE        GOL_Right_Equal
                    LDA        SE_SPRITE_X0,X       
                    CMP        SE_SPRITE_X0,Y
                    BLT        GOL_Right_Front
*--
GOL_Right_Behind    LDA        SE_SPRITE_X0,X       ; Opponent is behind
                    SEC
                    SBC        SE_SPRITE_X0,Y
                    CMP        #$0038
                    BGE        GOL_RB_Long
                    CMP        #$002A
                    BGE        GOL_RB_Middle
                    CMP        #$0015
                    BGE        GOL_RB_Close
*-
GOL_RB_Contact      LDA        SE_SPRITE_ID,Y       ; Opponent and Player are in contact
                    JSR        IsDuckSprite
                    BCS        GOL_RB_Contact_Down
                    LDA        SE_SPRITE_ID,Y 
                    JSR        IsFlightSprite
                    BCS        GOL_RB_Contact_Up
                    LDA        #$0004
                    JMP        GOL_End       
GOL_RB_Contact_Up   LDA        #$0007
                    JMP        GOL_End
GOL_RB_Contact_Down LDA        #$0001
                    JMP        GOL_End
GOL_RB_Close        LDA        #$0044               ; Opponent is close left
                    JMP        GOL_End
GOL_RB_Middle       LDA        #$0444               ; Opponent is middle far left
                    JMP        GOL_End
GOL_RB_Long         LDA        #$4444               ; Opponent is far left
                    JMP        GOL_End
*--
GOL_Right_Equal     LDA        SE_SPRITE_ID,Y       ; Player and Opponent share about the same position
                    JSR        IsDuckSprite
                    BCS        GOL_Right_Equal_Down
                    LDA        SE_SPRITE_ID,Y 
                    JSR        IsFlightSprite
                    BCS        GOL_Right_Equal_Up
                    LDA        #$0005               ; Opponent is stand-up
                    JMP        GOL_End
GOL_Right_Equal_Up  LDA        #$0008               ; Opponent is in the air
                    JMP        GOL_End
GOL_Right_Equal_Down LDA       #$0002               ; Opponent is in the ground
                    JMP        GOL_End
*--
GOL_Right_Front     LDA        SE_SPRITE_X0,Y       ; Opponent is in front
                    SEC
                    SBC        SE_SPRITE_X0,X
                    CMP        #$0038
                    BGE        GOL_RF_Long
                    CMP        #$002A
                    BGE        GOL_RF_Middle
                    CMP        #$0015
                    BGE        GOL_RF_Close
*-
GOL_RF_Contact      LDA        SE_SPRITE_ID,Y       ; Opponent and Player are in contact
                    JSR        IsDuckSprite
                    BCS        GOL_RF_Contact_Down
                    LDA        SE_SPRITE_ID,Y 
                    JSR        IsFlightSprite
                    BCS        GOL_RF_Contact_Up
                    LDA        #$0006
                    JMP        GOL_End       
GOL_RF_Contact_Up   LDA        #$0009
                    JMP        GOL_End
GOL_RF_Contact_Down LDA        #$0003
                    JMP        GOL_End
GOL_RF_Close        LDA        #$0066               ; Opponent is close right
                    JMP        GOL_End
GOL_RF_Middle       LDA        #$0666               ; Opponent is middle far right
                    JMP        GOL_End
GOL_RF_Long         LDA        #$6666               ; Opponent is far right
                    JMP        GOL_End
*----
GOL_Left            JSR        IsAboutSamePosition  ; Player goes Left
                    BNE        GOL_Left_Equal
                    LDA        SE_SPRITE_X0,X     
                    CMP        SE_SPRITE_X0,Y
                    BGE        GOL_Left_Front
*--
GOL_Left_Behind     LDA        SE_SPRITE_X0,Y       ; Opponent is behind
                    SEC
                    SBC        SE_SPRITE_X0,X
                    CMP        #$0038
                    BGE        GOL_LB_Long
                    CMP        #$002A
                    BGE        GOL_LB_Middle
                    CMP        #$0015
                    BGE        GOL_LB_Close
*-
GOL_LB_Contact      LDA        SE_SPRITE_ID,Y       ; Opponent and Player are in contact
                    JSR        IsDuckSprite
                    BCS        GOL_LB_Contact_Down
                    LDA        SE_SPRITE_ID,Y 
                    JSR        IsFlightSprite
                    BCS        GOL_LB_Contact_Up
                    LDA        #$0006
                    JMP        GOL_End       
GOL_LB_Contact_Up   LDA        #$0009
                    JMP        GOL_End
GOL_LB_Contact_Down LDA        #$0003
                    JMP        GOL_End
GOL_LB_Close        LDA        #$0066               ; Opponent is close left
                    JMP        GOL_End
GOL_LB_Middle       LDA        #$0666               ; Opponent is middle far left
                    JMP        GOL_End
GOL_LB_Long         LDA        #$6666               ; Opponent is far left
                    JMP        GOL_End
*--
GOL_Left_Equal      LDA        SE_SPRITE_ID,Y       ; Player and Opponent share about the same position
                    JSR        IsDuckSprite
                    BCS        GOL_Left_Equal_Down
                    LDA        SE_SPRITE_ID,Y 
                    JSR        IsFlightSprite
                    BCS        GOL_Left_Equal_Up
                    LDA        #$0005
                    JMP        GOL_End
GOL_Left_Equal_Up   LDA        #$0008
                    JMP        GOL_End
GOL_Left_Equal_Down LDA        #$0002
                    JMP        GOL_End
*--
GOL_Left_Front      LDA        SE_SPRITE_X0,X       ; Opponent is in front
                    SEC
                    SBC        SE_SPRITE_X0,Y
                    CMP        #$0038
                    BGE        GOL_LF_Long
                    CMP        #$002A
                    BGE        GOL_LF_Middle
                    CMP        #$0015
                    BGE        GOL_LF_Close
*-
GOL_LF_Contact      LDA        SE_SPRITE_ID,Y       ; Opponent and Player are in contact
                    JSR        IsDuckSprite
                    BCS        GOL_LF_Contact_Down
                    LDA        SE_SPRITE_ID,Y 
                    JSR        IsFlightSprite
                    BCS        GOL_LF_Contact_Up
                    LDA        #$0004
                    JMP        GOL_End       
GOL_LF_Contact_Up   LDA        #$0007
                    JMP        GOL_End
GOL_LF_Contact_Down LDA        #$0001
                    JMP        GOL_End
GOL_LF_Close        LDA        #$0044               ; Opponent is close right
                    JMP        GOL_End
GOL_LF_Middle       LDA        #$0444               ; Opponent is middle far right
                    JMP        GOL_End
GOL_LF_Long         LDA        #$4444               ; Opponent is far right                   
*----
GOL_End             RTS


********************************************************************************************************************************
*  IsAboutSamePosition() :  Check if both sprites share the same position (IN:X=Sprite1Index,IN:Y=Sprite2Index,OUT:A=1IfTrue). *
********************************************************************************************************************************  
IsAboutSamePosition LDA        SE_SPRITE_X0,X       ; Compute |Sprite1X0 - Sprite1X0|
                    CMP        SE_SPRITE_X0,Y
                    BEQ        IASP_True
                    BLT        IASP_SUB
                    SEC
                    SBC        SE_SPRITE_X0,Y
                    BRA        IASP_CMP 
IASP_SUB            LDA        SE_SPRITE_X0,Y
                    SEC
                    SBC        SE_SPRITE_X0,X  
IASP_CMP            CMP        #$0008               ; 16 Pixels
                    BGE        IASP_False
*-
IASP_True           LDA        #$0001
                    RTS
IASP_False          LDA        #$0000
                    RTS


*****************************************************************************************
*  HasSpaceBehind() :  Check if there is space behind (IN:X=SpriteIndex,OUT:A=1IfTrue). *
*****************************************************************************************  
HasSpaceBehind      LDA        SE_SPRITE_ID,X
                    JSR        GetSpriteDirection
                    BCS        HSB_Right
*--
HSB_Left            LDA        SE_SPRITE_X1,X       ; Sprite goes Left
                    CMP        #$0086
                    BLT        HSB_Yes
                    BRA        HSB_No
*--
HSB_Right           LDA        SE_SPRITE_X0,X       ; Sprite goes Right
                    CMP        #$0018
                    BGE        HSB_Yes
                    BRA        HSB_No
*--                    
HSB_Yes             LDA        #$0001               ; There is Space behind Sprite
                    RTS
*--
HSB_No              LDA        #$0000               ; There is NO Space behind Sprite
                    RTS
                    
                                    
***********************************************************************************    
*  GetLevelMoveWait() :  Compute delay between each move (IN:A=Level,OUT=A=Wait). *                    
***********************************************************************************
GetLevelMoveWait    PHA
                    LDA        PG_GameType          ; Game Type 0:CompVsComp
                    BNE        GLMW_0
                    PLA
                    LDA        #$0002               ; Demo Mode Comp Vs Comp
                    BRA        GLMW_End
*--
GLMW_0              PLA
                    CMP        #$000E               ; Adapt opponent delay based on level
                    BLT        GLMW_1
                    LDA        #$0000               ; Levels >= 14
                    BRA        GLMW_End
GLMW_1              CMP        #$000C
                    BLT        GLMW_2
                    LDA        #$0001               ; Levels 12-13
                    BRA        GLMW_End
GLMW_2              CMP        #$000A
                    BLT        GLMW_3
                    LDA        #$0002               ; Levels 10-11
                    BRA        GLMW_End
GLMW_3              CMP        #$0008
                    BLT        GLMW_4
                    LDA        #$0003               ; Levels 8-9
                    BRA        GLMW_End
GLMW_4              CMP        #$0006
                    BLT        GLMW_5
                    LDA        #$0004               ; Levels 6-7
                    BRA        GLMW_End
GLMW_5              CMP        #$0003
                    BLT        GLMW_6
                    LDA        #$0005               ; Levels 3-5
                    BRA        GLMW_End
GLMW_6              LDA        #$0006               ; Levels 1-2
*-
GLMW_End            RTS
                

**************************************************************************************
*  GetRandomIndex() :  Get a Random Index (IN:A=MaxIndex,IN:X=Sprite,OUT:A=Index*2). *
**************************************************************************************
GetRandomIndex      CMP        #$0002               ; Get a Ramdom value (1-100)
                    BEQ        GRI_2
                    CMP        #$0004
                    BEQ        GRI_4
                    CMP        #$0008
                    BEQ        GRI_8
                    CMP        #$0010
                    BEQ        GRI_16
                    LDA        #$0000               ; Other
                    BRA        GRI_End
*-                   
GRI_2               JSL        GetRandomNumber      ; 2 choices [0-1]
                    DEC
                    AND        #$0001
                    BRA        GRI_End
GRI_4               JSL        GetRandomNumber      ; 4 choices [0-3]
                    DEC
                    AND        #$0003
                    BRA        GRI_End
GRI_8               JSL        GetRandomNumber      ; 8 choices [0-7]
                    DEC
                    AND        #$0007
                    BRA        GRI_End
GRI_16              JSL        GetRandomNumber      ; 16 choices [0-F]
                    DEC
                    AND        #$000F
*-                                      
GRI_End             ASL
                    RTS


*----------  Global Tables                             
SHR_LINE_TABLE      HEX        0000,A000,4001,E001,8002,2003,C003,6004,0005,A005,4006,E006,8007,2008,C008,6009,000A,A00A,400B,E00B
                    HEX        800C,200D,C00D,600E,000F,A00F,4010,E010,8011,2012,C012,6013,0014,A014,4015,E015,8016,2017,C017,6018
                    HEX        0019,A019,401A,E01A,801B,201C,C01C,601D,001E,A01E,401F,E01F,8020,2021,C021,6022,0023,A023,4024,E024
                    HEX        8025,2026,C026,6027,0028,A028,4029,E029,802A,202B,C02B,602C,002D,A02D,402E,E02E,802F,2030,C030,6031
                    HEX        0032,A032,4033,E033,8034,2035,C035,6036,0037,A037,4038,E038,8039,203A,C03A,603B,003C,A03C,403D,E03D
                    HEX        803E,203F,C03F,6040,0041,A041,4042,E042,8043,2044,C044,6045,0046,A046,4047,E047,8048,2049,C049,604A
                    HEX        004B,A04B,404C,E04C,804D,204E,C04E,604F,0050,A050,4051,E051,8052,2053,C053,6054,0055,A055,4056,E056
                    HEX        8057,2058,C058,6059,005A,A05A,405B,E05B,805C,205D,C05D,605E,005F,A05F,4060,E060,8061,2062,C062,6063
                    HEX        0064,A064,4065,E065,8066,2067,C067,6068,0069,A069,406A,E06A,806B,206C,C06C,606D,006E,A06E,406F,E06F
                    HEX        8070,2071,C071,6072,0073,A073,4074,E074,8075,2076,C076,6077,0078,A078,4079,E079,807A,207B,C07B,607C
                                    
*------------------------ Other Source Files -----------------------------------

*-- Sprite Engine Functions 
                    PUT        AppleIIgsKarate.Engine  ; Sprite Engine Functions

*--  AI Functions
                    PUT        AppleIIgsKarate.AI      ; Artificial Intelligence Functions

*--  Data                                    
                    PUT        AppleIIgsKarate.Data    ; Sprite / Sequence / Erase / Collision data                                    

                    ASC        '- End of File -'

********************************************************************************
