******************************************************************************************
*                                                                                        *
*  AppleIIgsKarate.Shared.s :  Library for shared functions.                             *
*                                                                                        *
******************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  April 2022  *
******************************************************************************************

************************************************************
*  ToolInitialize() :  Initialize Tools layer (OUT:A=@DP). *
************************************************************
ToolInitialize      _TLStartUp                      ; Start Tools
                    PHA
                    _MMStartUp                      ; Start Memory Manager Tool Set
                    PLA
                    STA        myID                 ; Get current ID
*--
                    _MTStartUp                      ; Start Miscellaneous Tool Set
*--
                    PushLong   #0                   ; Allocate Page Direct in Bank 00
                    PushLong   #$000100
                    PushWord   myID
                    PushWord   #$C005               ; Fixed, Page Aligned, Locked, Unpurgeable
                    PushLong   #0
                    _NewHandle
                    PLX                             ; Handle Low  Address
                    PLA                             ; Handle High Address (00XX bank)
                    XBA
                    STA        TI_1+2
TI_1                LDAL       $000000,X            ; Get Low Address  A=??/XXXX
                    PHA
*--
                    _SoundStartUp                   ; Start Sound Tool Set
                    BCS        TI_NoSound
                    PushLong   #0
                    PushWord   #11
                    _GetVector
                    PullLong   BackupSoundVector
                    PushWord   #11
                    PushLong   #EntrySoundVector
                    _SetVector
                    BRA        TI_CompactMemory
TI_NoSound          INC        SoundToolError       ; No Sound Available                    
*--
TI_CompactMemory    PushLong   #0                   ; Compact Memory
                    PushLong   #$8FFFFF
                    PushWord   myID
                    PushWord   #%11000000_00000000
                    PushLong   #0
                    _NewHandle
                    _DisposeHandle
                    _CompactMem
*--
                    PushLong   #0                   ; Ask Shadowing Screen ($8000 bytes from $01/2000)
                    PushLong   #$8000
                    PushWord   myID
                    PushWord   #%11000000_00000011
                    PushLong   #$012000
                    _NewHandle
                    PLA
                    PLA
*--
                    RTS


************************************************
*  ToolTerminate() :  Free Memory, Stop Tools. *
************************************************
ToolTerminate       LDA        SoundToolError       ; Stop Tools
                    BNE        TT_MiscTool
                    PushWord   #11
                    PushLong   BackupSoundVector    ; Put back Sound Vector
                    _SetVector
                    _SoundShutDown
*--
TT_MiscTool         _MTShutDown
*--
                    PushWord   myID
                    _DisposeAll
*--               
                    PushWord   myID
                    _MMShutDown
*--               
                    _TLShutDown
*--
                    RTS
               
*----------   Tools Variables   ---------
myID                ds         2                    ; ID of this Program in memory
SoundToolError      HEX        0000                 ; 1 if sound Error
BackupSoundVector   DS         4                    ; Interrupt Sound Vector backup


********************************************************
*  BackupEnvironment() :  Backup Environment Settings. *
********************************************************
BackupEnvironment   SEP        #$20                 ; Backup Environment values (color, border...)
*--
                    LDAL       TBCOLOR              ; Text + Background Color
                    STA        BE_C022
*--            
                    LDAL       NEWVIDEO             ; Apple IIgs Graphic Mode
                    STA        BE_C029
*--            
                    LDAL       CLOCKCTL             ; Border Color
                    STA        BE_C034
*--            
                    LDAL       SHADOW               ; Shadowing
                    STA        BE_C035
*--
                    LDAL       RD80VID              ; 80 Column status
                    STA        BE_C01F
*--
                    REP        #$30
                    RTS


**********************************************************
*  RestoreEnvironment() :  Restore Environment Settings. *
**********************************************************
RestoreEnvironment  SEP        #$20                 ; Restore Environment values (color, border...)
*--
                    LDA        BE_C035              ; Shadowing
                    STAL       SHADOW
*--            
                    LDA        BE_C034              ; Border Color
                    STAL       CLOCKCTL
*--            
                    LDA        BE_C029              ; Apple IIgs Graphic Mode
                    STAL       NEWVIDEO
*--            
                    LDA        BE_C022              ; Text + Background Color
                    STAL       TBCOLOR
*--
                    LDA        BE_C01F              ; Restaure 80 Column if needed
                    ASL                             ; Bit #7=1 if 80 column
                    BCC        RE_End
                    STAL       SET80VID             ; Set 80 column mode      
*--
RE_End              REP        #$30
                    RTS

*--------------   Environment Variables   -------------
BE_C01F             HEX        00                   ; 80 Column status
BE_C022             HEX        00                   ; Background + Text Color
BE_C029             HEX        00                   ; Linearization of the Graphic Page
BE_C034             HEX        00                   ; Border Color
BE_C035             HEX        00                   ; Shadowing


***********************************************************************************
*  AllocOneBank() :  Allocate 1 Bank of 64 KB (OUT:A=XX/00,OUT:Carry=SetOnError). *
***********************************************************************************
AllocOneBank        PushLong   #0
                    PushLong   #$10000              ; 64 KB
                    PushWord   myID                 ; ID of my current Program
                    PushWord   #%11000000_00011100
                    PushLong   #0
                    _NewHandle
                    PLX                             ; Handle Low Address
                    PLA                             ; Handle High Address (00XX)
                    XBA
                    STA        AOB_1+2
AOB_1               LDAL       $000001,X            ; Get Bank Address (A=XX/00)
                    RTS


***************************************************************************************************************
*  MemoryAllocation() :  Allocate all required Banks of 64 KB (IN:A=MaxBank,IN:X=@TableBank,OUT:A=AllocBank). *
***************************************************************************************************************
MemoryAllocation    ASL                             ; Maximum of # Bank required x 2
                    STA        MA_3+1
                    STX        MA_2+1
                    LDX        #$0000
*--                    
MA_1                PHX                             ; Allocate one Bank of 64 KB
                    JSR        AllocOneBank
                    PLX
                    BCS        MA_End               ; Error, no more memory available
*--
MA_2                STA        $AAAA,X              ; Store Bank Number in the Table
                    INX
                    INX
*--                    
MA_3                CPX        #$AAAA               ; Don't allocate more than required
                    BMI        MA_1                
*---                    
MA_End              TXA                             ; Number of allocated Bank x 2
                    LSR      
                    RTS


**************************************************
*  ComputeGSSpeed() :  Compute Apple IIgs Speed. *
**************************************************
ComputeGSSpeed      LDA        #$0010               ; SHR Line 16
                    JSL        WaitForSpot
*--                 
                    STZ        CGSS_Counter   
                    LDA        #$0020               ; SHR Line 32
                    JSL        WaitForSpot
*--
ComputeGSSpeed_1    INC        CGSS_Counter         ; Wait for 32 lines
                    LDA        #$0040               ; SHR Line 64
                    JSR        CheckForSpot
                    BNE        ComputeGSSpeed_1
*--
                    LDA        CGSS_Counter
                    CMP        #$0040
                    BLT        CGSS_1_Mhz           ; 1 Mhz Apple IIgs
                    CMP        #$0070
                    BLT        CGSS_2_Mhz           ; 2.8 Mhz Apple IIgs
                    CMP        #$0400
                    BLT        CGSS_8_Mhz           ; Accerated Apple IIgs (7Mhz-16Mhz)         
*--
CGSS_U_Mhz          LDA        #$0003
                    BRA        CGSS_Code
CGSS_8_Mhz          LDA        #$0002
                    BRA        CGSS_Code
CGSS_2_Mhz          LDA        #$0001
                    BRA        CGSS_Code
CGSS_1_Mhz          LDA        #$0000
*--
CGSS_Code           STA        CGSS_SpeedCode
                    RTS  

*--------------   ComputeGSSpeed Variables   -------------
CGSS_Counter        HEX        0000                 ; 0820 = Full Speed Emulator, 0129 = 8Mhz, 005B = 2,8 Mhz, 0026 = 1 Mhz
CGSS_SpeedCode      HEX        0000                 ; Speed Code : 0=1Mhz, 1=2.8Mhz, 2=Accelerator Card, 3=Emulator Unlimited


********************************************************
*  WaitForKey() :  Wait for a Key pressed (OUT:A=Key). *
********************************************************
WaitForKey          ENT                             ; Wait for a Key Press
                    SEP        #$20                 
WFK_1               LDAL       KBDREAD
                    BPL        WFK_1
                    STAL       KBDSTRB
                    REP        #$30
                    AND        #$007F               ; Clear Bit #7
                    RTL


************************************************************
*  WaitForCtrlKey() :  Wait for Control/Shift Key pressed. *
************************************************************
WaitForCtrlKey      SEP        #$20                 ; Wait for a Control/Shift Key Press
                    LDA        WFCK_Source
                    BNE        WFCK_2
WFCK_1              LDAL       KEYMODREG          
                    BIT        #$02                 ; Control
                    BEQ        WFCK_1
                    INC        WFCK_Source
                    BRA        WFCK_End
WFCK_2              LDAL       KEYMODREG           
                    BIT        #$01                 ; Shift
                    BEQ        WFCK_2
                    STZ        WFCK_Source
WFCK_End            REP        #$30
                    RTS
*----------   WaitForCtrlKey Variables   ---------
WFCK_Source         HEX        00                   ; 0:Wait For Ctrl/1:Wait for Shift


***********************************************************
*  IsKeyPressed() :  Check for a Key pressed (OUT:A=Key). *
***********************************************************
IsKeyPressed        SEP        #$20                 ; Wait for a Key Press
                    LDAL       KBDREAD
                    BPL        IKP_1
                    STAL       KBDSTRB
                    BRA        IKP_2
IKP_1               LDA        #$00                 ; No Key
IKP_2               REP        #$30
                    AND        #$007F               ; Clear Bit #7
                    RTS


*********************************************************************************
*  CheckForKeyAction() :  Check for a Key pressed / Joystick (OUT:A=Direction). *
*********************************************************************************
CheckForKeyAction   SEP        #$20                 ; Check Keyboard Events
*--
CFKA_Button0        LDAL       KEYMODREG            ; Special Key
                    BIT        #$82                 ; Bit 7 : Apple Key / Bit 1 : Control Key
                    BEQ        CFKA_Button1
*-
CFKA_Button1        LDAL       KEYMODREG            ; Special Key
                    BIT        #$41                 ; Bit 6 : Option Key / Bit 0 : Shift Key
                    BEQ        CFKA_Keyboard                    
*--
CFKA_Keyboard       LDAL       KBDREAD              ; Read Keyboard
                    BPL        CFKA_NoKey
                    STAL       KBDSTRB
                    AND        #$7F                 ; Clear Bit #7
*--
                    CMP        #$30                 ; Action '0'
                    BEQ        CFKA_Action
                    CMP        #$20                 ; Space
                    BEQ        CFKA_Action
                    CMP        #$0D                 ; Return
                    BEQ        CFKA_Action
*-
                    CMP        #$0B                 ; Arrow Up
                    BEQ        CFKA_Up
                    CMP        #$0A                 ; Arrow Down
                    BEQ        CFKA_Down
                    CMP        #$08                 ; Arrow Left
                    BEQ        CFKA_Left
                    CMP        #$15                 ; Arrow Rigth
                    BEQ        CFKA_Right
                    BRA        CFKA_NoKey
*--
CFKA_Action         LDA        #$30                 ; Action : '0'
                    BRA        CFKA_End
*-
CFKA_Up             LDA        #$38                 ; Up = '8'
                    BRA        CFKA_End
*-
CFKA_Down           LDA        #$32                 ; Down = '2'
                    BRA        CFKA_End
*-
CFKA_Left           LDA        #$34                 ; Left = '4'
                    BRA        CFKA_End
*-
CFKA_Right          LDA        #$36                 ; Left = '6'
                    BRA        CFKA_End
*--
CFKA_NoKey          LDAL       #$00                 ; No Key 
CFKA_End            REP        #$30
                    AND        #$00FF
                    RTS


***********************************************
*  ClearKDBStrobe() :  Clear Keyboard Strobe. *
***********************************************
ClearKDBStrobe      ENT
                    SEP        #$20                 ; Reset any Key Pressed
                    STAL       KBDSTRB
                    REP        #$30
                    RTL


*************************************************************************
*  HasKeyPressed() :  Check for a Key pressed (OUT:Carry=SetIfPressed). *
*************************************************************************
HasKeyPressed       SEP        #$20                 ; Wait for a Key Press
                    CLC                             ; Init as 'Not Pressed'
                    LDAL       KBDREAD
                    BPL        HKP_End
                    STAL       KBDSTRB
                    SEC
HKP_End             REP        #$30
                    RTS

*******************************************************************
*  GetJoystickInput() :  Get Apple Joystick Inputs (OUT:A=Input). *
*******************************************************************
GetJoystickInput    ENT
                    PHB
                    PHK
                    PLB
*--
GAJI_Joystick       JSL        ReadAppleJoystick    ; Joystick Centered = 5
                    STA        GAJI_End_1+1
                    SEP        #$20
*--
GAJI_Button_0       LDAL       BUTN0                ; Button 0 / Open Apple           
                    BIT        #$80
                    BEQ        GAJI_Button_1
                    INC        GAJI_End_1+2         ; Button 0 is pushed
GAJI_Button_1       LDAL       BUTN1                ; Button 1 / Option
                    BIT        #$80                 
                    BEQ        GAJI_End
                    LDA        #$02
                    TSB        GAJI_End_1+2         ; Button 1 is pushed
*--
GAJI_End            REP        #$30
GAJI_End_1          LDA        #$AAAA
                    PLB
                    RTL


***********************************************************************
*  GetJoypad1Input() :  Get Joypad #1 Inputs (IN:X=Slot,OUT:A=Input). *
***********************************************************************
GetJoypad1Input     ENT
                    PHB
                    PHK
                    PLB
                    LDA        #$0035               ; Init (No Button|Centered)
                    STA        GJ1I_Value+1
*--
GJ1I_NoCard         CPX        #$0000               ; No Card found
                    BEQ        GJ1I_Value
*--
GJ1I_ReadStick      SEP        #$20                 ; Read in 8 bit
                    LDAL       $00C000,X
                    REP        #$20
                    PHA
                    AND        #$000F
                    TAY
                    LDA        GJ1I_Table,Y
                    AND        #$00FF
                    STA        GJ1I_Value+1
*--
GJ1I_ReadButtons    PLA
                    AND        #$00C0
                    BIT        #$0080               ; Button 0
                    BEQ        GJ1I_Button_1
                    INC        GJ1I_Value+2         ; Button 0 is pushed
GJ1I_Button_1       BIT        #$0040               ; Button1
                    BEQ        GJ1I_Value
                    LDA        #$0002
                    TSB        GJ1I_Value+2         ; Button 1 is pushed
*--
GJ1I_Value          LDA        #$AAAA               ; Buttons | Stick
GJ1I_End            PLB
                    RTL

*----------   GetJoypad1Input Variables   ---------
GJ1I_Table          HEX        35,38,32,35,34,37,31,35
                    HEX        36,39,33,35,35,35,35,35


***********************************************************************
*  GetJoypad2Input() :  Get Joypad #2 Inputs (IN:X=Slot,OUT:A=Input). *
***********************************************************************
GetJoypad2Input     ENT
                    PHB
                    PHK
                    PLB
                    LDA        #$0035               ; Init (No Button|Centered)
                    STA        GJ2I_Value+1
*--
GJ2I_NoCard         CPX        #$0000               ; No Card found
                    BEQ        GJ2I_Value
*--
GJ2I_ReadStick      SEP        #$20                 ; Read in 8 bit
                    LDAL       $00C001,X
                    REP        #$20
                    PHA
                    AND        #$000F
                    TAY
                    LDA        GJ2I_Table,Y
                    AND        #$00FF
                    STA        GJ2I_Value+1
*--
GJ2I_ReadButtons    PLA
                    AND        #$00C0
                    BIT        #$0080               ; Button 0
                    BEQ        GJ2I_Button_1
                    INC        GJ2I_Value+2         ; Button 0 is pushed
GJ2I_Button_1       BIT        #$0040               ; Button1
                    BEQ        GJ2I_Value
                    LDA        #$0002
                    TSB        GJ2I_Value+2         ; Button 1 is pushed
*--
GJ2I_Value          LDA        #$AAAA               ; Buttons | Stick
GJ2I_End            PLB
                    RTL

*----------   GetJoypad2Input Variables   ---------
GJ2I_Table          HEX        35,38,32,35,34,37,31,35
                    HEX        36,39,33,35,35,35,35,35


*************************************************************
*  GetJoypadSlot() :  Get Four Play Card Slot (OUT:A=Slot). *
*************************************************************
GetJoypadSlot       ENT
                    PHB
                    PHK
                    PLB
*--
                    STZ        GJS_Found+1          ; Init
                    LDX        #$00F0
GJS_Loop            SEP        #$20
                    LDAL       $00C000,X
                    CMP        #$20                 ; $20 for Rev B, $60 for Rev A
                    BNE        GJS_Next
                    LDAL       $00C001,X
                    CMP        #$20
                    BNE        GJS_Next
                    LDAL       $00C002,X
                    CMP        #$20
                    BNE        GJS_Next
                    LDAL       $00C003,X
                    CMP        #$20
                    BNE        GJS_Next
                    STX        GJS_Found+1          ; Slot # [Slot #7 00F0-0090 Slot #1]
                    BRA        GJS_End
GJS_Next            REP        #$20
                    TXA
                    SEC
                    SBC        #$0010
                    TAX
                    CPX        #$0080
                    BNE        GJS_Loop
GJS_End             REP        #$20
GJS_Found           LDA        #$0000     
*--
                    PLB               
                    RTL


*************************************************************
*  GetKeyboardInput() :  Get Keyboard Inputs (OUT:A=Input). *
*************************************************************
GetKeyboardInput    ENT
                    PHB
                    PHK
                    PLB
*--
                    LDA        #$0035               ; Joystick Centered = 5
                    STA        GKI_End_1+1
                    SEP        #$20
*--
GKI_Button          LDAL       KEYMODREG            ; Button 0 / 1           
                    BIT        #$02                 ; Control
                    BEQ        GKI_Button_1
                    INC        GKI_End_1+2          ; Button 0 is pushed
GKI_Button_1        BIT        #$01                 ; Shift
                    BEQ        GKI_Keyboard
                    LDA        #$02
                    TSB        GKI_End_1+2          ; Button 1 is pushed
*--
GKI_Keyboard        LDAL       KBDREAD              ; Keyboard
                    BPL        GKI_End
                    AND        #$7F
                    CMP        #$31
                    BLT        GKI_End
                    CMP        #$3A
                    BGE        GKI_End
                    STA        GKI_End_1+1          ; Direction 1-9
                    STAL       KBDSTRB              ; Clear Strobe only if we have pulled the input
*--
GKI_End             REP        #$30
GKI_End_1           LDA        #$0000
                    PLB
                    RTL


****************************************************************************
*  ReadAppleJoystick() :  Read Apple Joystick coordinates (OUT:A=[31-39]). *
****************************************************************************
ReadAppleJoystick   ENT
                    SEI                             ; No More Interrupts
                    PHD                             ; Backup Direct Page Address
                    PEA        $0000                ; Make some space on Stack to store JoyX and JoyY
                    LDA        #$C000
                    TCD                             ; Direct Page to $C000         
                    LDA        #$8000
                    TRB        $36-1                ; Turn as 1 Mhz (CYAREG=$C036)
*--
                    LDA        $70                  ; Reset Paddle (PTRIG=$C070)
                    LDX        #$0090
RAJ_Loop            LDA        $64                  ; Read X Paddle 0 (PADDL0=$C064) + Paddle 1 (PADDL1=$C065)
                    AND        #$8080               ; Paddle 1 | Paddle 0
                    BEQ        RAJ_Digital_X
                    ASL                             ; Paddle 0 | Paddle 1
                    ADC        $01,S                ; Axis X | Axis Y [00-3C-79]
                    STA        $01,S
                    DEX
                    BNE        RAJ_Loop
*-
RAJ_NoJoystick      LDA        #$3535               ; No Joystick available => Force 3C,3C
                    STA        $01,S
*--
RAJ_Digital_X       LDA        $01,S                ; Turn Analog X into Digital [77-50],[4F-28],[27-00]
                    STAL       $000300
                    XBA
                    AND        #$00FF
                    CMP        #$0028
                    BLT        RAJ_Left
                    CMP        #$0050
                    BLT        RAJ_Middle_X
RAJ_Right           LDA        #$0036  
                    BRA        RAJ_Digital_X_End
RAJ_Middle_X        LDA        #$0035
                    BRA        RAJ_Digital_X_End
RAJ_Left            LDA        #$0034
RAJ_Digital_X_End   TAY
*--
RAJ_Digital_Y       LDA        $01,S                ; Turn Analog Y into Digital [77-50],[4F-28],[27-00]
                    AND        #$00FF
                    CMP        #$0028
                    BLT        RAJ_Up
                    CMP        #$0050
                    BGE        RAJ_Down
RAJ_Middle_Y        TYA        
                    BRA        RAJ_Digital_Y_End
RAJ_Down            TYA
                    SEC
                    SBC        #$0003 
                    BRA        RAJ_Digital_Y_End
RAJ_Up              TYA
                    CLC
                    ADC        #$0003
RAJ_Digital_Y_End   TAY                                                               
*--
RAJ_End             LDA        #$8000
                    TSB        $36-1                ; Full Speed (CYAREG=$C036)
                    PLA                             ; Clear Stack Space
                    PLD                             ; Restore Direct Page
                    CLI                             ; Interrupts are back
                    TYA
                    RTL


*******************************************************************************************
*  HasAppleJoystick() :  Check if Apple Analog Joystick is connected (OUT:A=0001IfThere). *
*******************************************************************************************
HasAppleJoystick    ENT
                    LDY        #$0001               ; Init = Has Joystick
                    SEI                             ; No More Interrupts
                    PHD                             ; Backup Direct Page Address
                    PEA        $0000                ; Make some space on Stack to store JoyX and JoyY
                    LDA        #$C000
                    TCD                             ; Direct Page to $C000         
                    LDA        #$8000
                    TRB        $36-1                ; Turn as 1 Mhz (CYAREG=$C036)
*--
                    LDA        $70                  ; Reset Paddle (PTRIG=$C070)
                    LDX        #$0090
HAJ_Loop            LDA        $64                  ; Read X Paddle 0 (PADDL0=$C064) + Paddle 1 (PADDL1=$C065)
                    AND        #$8080               ; Paddle 1 | Paddle 0
                    BEQ        HAJ_End
                    ASL                             ; Paddle 0 | Paddle 1
                    ADC        $01,S                ; Axis X | Axis Y [00-3C-79]
                    STA        $01,S
                    DEX
                    BNE        HAJ_Loop
*-
HAJ_NoJoystick      LDY        #$0000               ; No Joystick available                        
*--
HAJ_End             LDA        #$8000
                    TSB        $36-1                ; Full Speed (CYAREG=$C036)
                    PLA                             ; Clear Stack Space
                    PLD                             ; Restore Direct Page
                    CLI                             ; Interrupts are back
                    RTL


*****************************************************
*  ResetTimerSec() :  Start the Timer (in Seconds). *
*****************************************************
ResetTimerSec       PHA                             ; Space for result
                    PHA
                    PHA
                    PHA
                    _ReadTimeHex                    ; Read current Time/Date in HEX Format
                    PLA
                    STA        RTS_SECOND_MINUTE
                    PLA
                    STA        RTS_HOUR_YEAR
                    PLA
                    STA        RTS_DAY_MONTH
                    PLA
                    STA        RTS_NULL_WEEKDAY                                                
                    RTS

*----------   ResetTimerSec Variables   ---------
RTS_SECOND_MINUTE   HEX        00,00                ; 0-59  0-59
RTS_HOUR_YEAR       HEX        00,00                ; 0-23  Year-1900  
RTS_DAY_MONTH       HEX        00,00                ; 0-11  0-30
RTS_NULL_WEEKDAY    HEX        00,00                ; 1(Sunday)-7  NULL


*********************************************************************************
*  GetTimerSec() :  Compute the number of seconds since the Reset (OUT:A=#Sec). *
*********************************************************************************
GetTimerSec         PHA                             ; Space for result
                    PHA
                    PHA
                    PHA
                    _ReadTimeHex                    ; Read current Time/Date in HEX Format        
                    PLA
                    STA        GTS_SECOND_MINUTE
                    PLA
                    PLA
                    PLA
*--
                    LDA        GTS_SECOND_MINUTE    ; Compute difference
                    CMP        RTS_SECOND_MINUTE
                    BEQ        GetTimerSec_Same
*--
                    SEP        #$20
                    LDA        GTS_SECOND_MINUTE
                    CMP        RTS_SECOND_MINUTE
                    BMI        GetTimerSec_Min
                    SEC
                    SBC        RTS_SECOND_MINUTE
                    BRA        GetTimerSec_End  
*--                    
GetTimerSec_Min     LDA        #$3C                 ; 60 sec
                    SEC
                    SBC        RTS_SECOND_MINUTE
                    CLC
                    ADC        GTS_SECOND_MINUTE
*--                    
GetTimerSec_End     REP        #$30
                    AND        #$00FF               ; 0-59 sec 
                    RTS
*--
GetTimerSec_Same    LDA        #$0000               ; In the Same second
                    RTS
                    
*----------   GetTimerSec Variables   ---------
GTS_SECOND_MINUTE   HEX        00,00                ; 0-59  0-59


*********************************************************************************************************
*  LoadFile() :  Load one File into memory (IN:X=@FilePath,IN:A=BankXX/00,OUT:A=Size,OUT:Carry=Status). *
*********************************************************************************************************
LoadFile            ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--            
                    STA        LF_gsosREAD+5        ; A = Target Bank XX/00
                    LDA        $04,S                ; Get Caller's Bank  A=??BB
                    AND        #$00FF
                    STA        LF_gsosOPEN+6        ; File Path Bank
                    STX        LF_gsosOPEN+4        ; X = File Path @ 
*--
LF_Open             JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       LF_gsosOPEN          ; Parameters Table Address (Open in Read Mode)
                    BCS        LF_ErrorOpen
                    LDA        LF_gsosOPEN+2
                    STA        LF_gsosGETEOF+2
                    STA        LF_gsosREAD+2
                    STA        LF_gsosCLOSE+2
*--
LF_GetSize          JSL        GSOS                 ; Get File Size
                    dw         $2019                ; Code for command 'GetFileSize'
                    adrl       LF_gsosGETEOF        ; Parameters Table Address
                    BCS        LF_ErrorGetSize
                    LDA        LF_gsosGETEOF+4
                    STA        LF_gsosREAD+8
                    LDA        LF_gsosGETEOF+6
                    STA        LF_gsosREAD+10
*--
LF_Read             JSL        GSOS                 ; Read File Content
                    dw         $2012                ; Code for command 'Read'
                    adrl       LF_gsosREAD          ; Parameters Table Address
                    BCS        LF_ErrorRead
*--
LF_Close            JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       LF_gsosCLOSE         ; Parameters Table Address
*--
LF_End              CLC                             ; No Error
                    LDA        LF_gsosGETEOF+4      ; A = File Size
                    PLB
                    RTL
*----
LF_Error            JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       LF_gsosCLOSE         ; Parameters Table Address
*--
LF_ErrorOpen        LDA        #$0001               ; Error Open
                    SEC
                    PLB
                    RTL
*--
LF_ErrorGetSize     LDA        #$0002               ; Error Get Size
                    SEC
                    PLB
                    RTL
*--
LF_ErrorRead        LDA        #$0003               ; Error Read
                    SEC
                    PLB
                    RTL
                         
*----------   LoadFile Variables   ---------
LF_gsosOPEN         dw         2                    ; Number of parameters (Open in Read Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       BS_FileData001       ; +4 File Pathname (init with an existing file path, so we only have to change 2 bytes)

LF_gsosGETEOF       dw         2                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
                    ds         4                    ; +2 eof

LF_gsosREAD         dw         4                    ; Number of parameters
                    ds         2                    ; +2  File Descriptor
                    ds         4                    ; +4  Data Buffer Address
                    ds         4                    ; +8  requestCount
                    ds         4                    ; +12 transferCount
                    
LF_gsosCLOSE        dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor


*******************************************************************************************
*  IsFileExist() :  Check if a file already exist on disk (IN:A=FilePath,OUT:A=1IfExist). *
*******************************************************************************************
IsFileExist         STA        IFE_gsosOPEN+4       ; A = File Path 
*--
IFE_Open            JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       IFE_gsosOPEN         ; Parameters Table Address (Open in Read Mode)
                    BCS        IFE_ErrorEnd
                    LDA        IFE_gsosOPEN+2
                    STA        IFE_gsosCLOSE+2
*--
IFE_Close           JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       IFE_gsosCLOSE        ; Parameters Table Address
*--
IFE_End             LDA        #$0001               ; File exist
                    RTS
*--
IFE_ErrorEnd        LDA        #$0000               ; File doesn't exist
                    RTS

*----------   IsFileExist Variables   ---------
IFE_gsosOPEN        dw         2                    ; Number of parameters (Open in Read Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       BS_FileData001       ; +4 File Pathname (init with an existing file path, so we only have to change 2 bytes)
                    
IFE_gsosCLOSE       dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor


***********************************************************************
*  ExitToLauncher() :  Quit program and go back to Finder / Launcher. *
***********************************************************************
ExitToLauncher      JSL        GSOS                 ; Quit Program
                    dw         $2029
                    adrl       gsosQUIT               
               
*----------   GS/OS Calls Variables   ---------
gsosQUIT            dw         2                    ; Number of parameters
                    ds         4                    ; +2 pathname
                    ds         2                    ; +6 flags


********************************************************************************************
*  CreateLogFile() :  Create the Log file on disk (IN:A=LogFilePath,OUT:Carry=SetOnError). *
********************************************************************************************
CreateLogFile       STA        CLF_gsosCREATE+2     ; A = File Path 
                    STA        CLF_gsosDESTROY+2
                    STA        ALL_LogFilePath
*----
                    JSL        GSOS                 ; Delete File
                    dw         $2002                ; Code for command 'Destroy'
                    adrl       CLF_gsosDESTROY      ; Parameters Table Address   
*----                    
                    SEP        #$20
                    LDA        #$E3
                    STA        CLF_gsosCREATE+6     ; File Access
                    LDA        #$0004
                    STA        CLF_gsosCREATE+8     ; File Type (Text)
                    LDA        #$01
                    STA        CLF_gsosCREATE+14    ; Storage Type (Standard)
                    REP        #$30
                    STZ        CLF_gsosCREATE+10    ; File AuxType
                    STZ        CLF_gsosCREATE+12
*--
                    JSL        GSOS                 ; Create File
                    dw         $2001                ; Code for command 'Create'
                    adrl       CLF_gsosCREATE       ; Parameters Table Address                    
*----
                    RTS               

*----------   CreateLogFile Variables   ---------
LogFilePath         strl       '1/LogFile.txt'
LOG_LINE_1          ASC        'Log Line #1',00
LOG_LINE_2          ASC        'Log Line #2',00
LOG_LINE_3          ASC        'Log Line #3',00
LOG_LINE_4          ASC        'Log Line %1Y-%2Y-%4Y 123',00

STRING_1            ASC        'SubString1',00
STRING_2            ASC        'SubString2',00
VALUEHEX            HEX        0020E1FF

CLF_gsosDESTROY     dw         1                    ; Number of parameters
                    adrl       LogFilePath          ; +2  File Pathname (init with an existing file path, so we only have to change 2 bytes)
                    
CLF_gsosCREATE      dw         5                    ; Number of parameters
                    adrl       LogFilePath          ; +2  File Pathname (init with an existing file path, so we only have to change 2 bytes)
                    ds         2                    ; +6  File Access
                    ds         2                    ; +8  File Type
                    ds         4                    ; +10 File AuxType       
                    ds         2                    ; +14 Storage Type


**************************************************************************************************************
*  AddLogLine() :  Add one line to the Log file (IN:A=LogLine,IN:X=ParamX,IN:Y=ParamY,OUT:Carry=SetOnError). *
**************************************************************************************************************
AddLogLine          STA        ALL_Prepare_1+1      ; A = @ Line
                    STA        ALL_PrepareParam+1
                    STA        ALL_PrepareNum_1+1
                    STA        ALL_PrepareNum_2+1
                    STA        ALL_PrepareNum_4+1
                    STA        ALL_PrepareString+1
*-
                    STX        ALL_PrepareX_1+1     ; X Parameter for Log Line
                    STX        ALL_PrepareNum1X+1
                    STX        ALL_PrepareNum2X_1+1
                    STX        ALL_PrepareNum4X_1+1
                    STX        ALL_PrepareStrX_1+1
                    INX
                    INX
                    STX        ALL_PrepareNum4X_2+1
*-
                    STY        ALL_PrepareY_1+1     ; Y Parameter for Log Line
                    STY        ALL_PrepareNum1Y+1
                    STY        ALL_PrepareNum2Y_1+1
                    STY        ALL_PrepareNum4Y_1+1
                    STY        ALL_PrepareStrY_1+1
                    INY
                    INY
                    STY        ALL_PrepareNum4Y_2+1
                                                  
*----  Get Current Time / Date  ----
ALL_GetTimeDate     PushLong   #ALL_TimeBuffer      ; Get current Time/Date
                    _ReadAsciiTime
                    LDX        #$0012               ; Move High Ascii to Ascii
ALL_GetTimeDate_1   LDA        ALL_TimeBuffer,X
                    AND        #$7F7F
                    STA        ALL_TimeBuffer,X
                    DEX
                    DEX
                    BPL        ALL_GetTimeDate_1
*-
ALL_CleanTimeDate   SEP        #$30                 ; Replace Space with 0
                    LDX        #$30                 ; '0'
                    LDA        ALL_TimeBuffer
                    CMP        #$20                 ; ' '
                    BNE        ALL_CleanTimeDate_1
                    STX        ALL_TimeBuffer
ALL_CleanTimeDate_1 LDA        ALL_TimeBuffer+3
                    CMP        #$20                 ; ' '
                    BNE        ALL_CleanTimeDate_2
                    STX        ALL_TimeBuffer+3           
ALL_CleanTimeDate_2 REP        #$30

*----  Prepare Log Line (Replace X and Y parameters, Compute Length)  ----
                    LDX        #$0000               ; Replace Log Line parameters
                    LDY        #$0000               ; X=Src, Y=Dst
*--
ALL_Prepare_0       SEP        #$20 
ALL_Prepare_1       LDA        $AAAA,X
                    INX
                    CMP        #$00
                    BEQ        ALL_GotoEnd          ; \0 is the End of Line
                    CMP        #$25                 ; '%'
                    BEQ        ALL_PrepareParam        
                    STA        ALL_LogBuffer,Y
                    INY
                    BRA        ALL_Prepare_1
ALL_GotoEnd         JMP        ALL_PrepareEnd
*----                             
ALL_PrepareParam    LDA        $AAAA,X              ; %1X|Y %2X|Y %4X|Y %SX %SX %X %Y
                    INX
                    CMP        #$31                 ; '1'
                    BEQ        ALL_PrepareNum_1
                    CMP        #$32                 ; '2'
                    BEQ        ALL_PrepareNum_2
                    CMP        #$34                 ; '4'
                    BEQ        ALL_GotoPrepNum_4
                    CMP        #$73                 ; 's'
                    BEQ        ALL_GotoPrepString
                    CMP        #$53                 ; 'S'
                    BEQ        ALL_GotoPrepString
                    CMP        #$58                 ; 'X'
                    BEQ        ALL_PrepareX 
                    CMP        #$59                 ; 'Y'
                    BEQ        ALL_PrepareY
*--
                    STA        ALL_LogBuffer+1,Y    ; Unknown % sequence
                    LDA        #$25                 ; '%'
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
                    BRA        ALL_Prepare_1
ALL_GotoPrepNum_4   JMP        ALL_PrepareNum_4
ALL_GotoPrepString  JMP        ALL_PrepareString
*----
ALL_PrepareX        REP        #$30                 ; %X : Insert X Data 16 bit
ALL_PrepareX_1      LDA        #$BBBB
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex
                    JSR        ALL_InsertHexaData2
                    BRA        ALL_Prepare_0
*----
ALL_PrepareY        REP        #$30                 ; %Y : Insert Y Data 16 bit
ALL_PrepareY_1      LDA        #$CCCC
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex
                    JSR        ALL_InsertHexaData2                  
                    BRA        ALL_Prepare_0
*----                 
                    mx         %10
ALL_PrepareNum_1    LDA        $AAAA,X              ; %1 : Insert 8 bit from Address X / Y
                    INX
                    CMP        #$59                 ; 'Y'
                    BEQ        ALL_PrepareNum1Y
                    CMP        #$58                 ; 'X'
                    BNE        ALL_PrepareNumUnk_1
*--     
ALL_PrepareNum1X    LDA        $BBBB                ; '%1X' : Copy 1 byte from address [X]
                    STA        ALL_ParamValueHex 
                    REP        #$30
                    JSR        ALL_InsertHexaData1  ; Add 1 byte in the String                    
                    JMP        ALL_Prepare_0
*--     
ALL_PrepareNum1Y    LDA        $CCCC                ; '%1Y' : Copy 1 byte from address [Y]
                    STA        ALL_ParamValueHex 
                    REP        #$30
                    JSR        ALL_InsertHexaData1  ; Add 1 byte in the String                    
                    JMP        ALL_Prepare_0
*--
                    mx         %10
ALL_PrepareNumUnk_1 STA        ALL_LogBuffer+2,Y    ; Unknown Sequence %1 sequence
                    LDA        #$25                 ; '%'
                    STA        ALL_LogBuffer,Y
                    LDA        #$31                 ; '1'
                    STA        ALL_LogBuffer+1,Y
                    INY
                    INY
                    INY
                    JMP        ALL_Prepare_1
*----
ALL_PrepareNum_2    LDA        $AAAA,X              ; %2 : Insert 16 bit from Address X / Y
                    INX
                    CMP        #$59                 ; 'Y'
                    BEQ        ALL_PrepareNum2Y
                    CMP        #$58                 ; 'X'
                    BNE        ALL_PrepareNumUnk_2
*--     
ALL_PrepareNum2X    REP        #$30                 ; '%2X' : Copy 2 bytes from address [X]
ALL_PrepareNum2X_1  LDA        $BBBB                
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex 
                    JSR        ALL_InsertHexaData2  ; Add 2 bytes in the String                    
                    JMP        ALL_Prepare_0
*--     
ALL_PrepareNum2Y    REP        #$30                 ; '%2Y' : Copy 2 bytes from address [Y]
ALL_PrepareNum2Y_1  LDA        $CCCC
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex 
                    JSR        ALL_InsertHexaData2  ; Add 2 bytes in the String                    
                    JMP        ALL_Prepare_0
*--
                    mx         %10
ALL_PrepareNumUnk_2 STA        ALL_LogBuffer+2,Y    ; Unknown Sequence %2 sequence
                    LDA        #$25                 ; '%'
                    STA        ALL_LogBuffer,Y
                    LDA        #$32                 ; '2'
                    STA        ALL_LogBuffer+1,Y
                    INY
                    INY
                    INY
                    JMP        ALL_Prepare_1
*----
ALL_PrepareNum_4    LDA        $AAAA,X              ; %4 : Insert 32 bit from Address X / Y
                    INX
                    CMP        #$59                 ; 'Y'
                    BEQ        ALL_PrepareNum4Y
                    CMP        #$58                 ; 'X'
                    BNE        ALL_PrepareNumUnk_4
*--     
ALL_PrepareNum4X    REP        #$30                 ; '%4X' : Copy 4 bytes from address [X]
ALL_PrepareNum4X_1  LDA        $BBBB            
                    XBA                             ; -> Big Endian    
                    STA        ALL_ParamValueHex+2 
ALL_PrepareNum4X_2  LDA        $BBBB                ; [X+2]
                    XBA                             ; -> Big Endian    
                    STA        ALL_ParamValueHex                     
                    JSR        ALL_InsertHexaData4  ; Add 4 bytes in the String                    
                    JMP        ALL_Prepare_0
*--     
ALL_PrepareNum4Y    REP        #$30                 ; '%4Y' : Copy 4 bytes from address [Y]
ALL_PrepareNum4Y_1  LDA        $CCCC
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex+2
ALL_PrepareNum4Y_2  LDA        $CCCC                ; [Y+2]
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex                     
                    JSR        ALL_InsertHexaData4  ; Add 4 bytes in the String                    
                    JMP        ALL_Prepare_0
*--
                    mx         %10
ALL_PrepareNumUnk_4 STA        ALL_LogBuffer+2,Y    ; Unknown Sequence %4 sequence
                    LDA        #$25                 ; '%'
                    STA        ALL_LogBuffer,Y
                    LDA        #$34                 ; '4'
                    STA        ALL_LogBuffer+1,Y
                    INY
                    INY
                    INY
                    JMP        ALL_Prepare_1
*----                 
ALL_PrepareString   LDA        $AAAA,X              ; %S : Insert String from Address X / Y
                    INX
                    CMP        #$59                 ; 'Y'
                    BEQ        ALL_PrepareStrY
                    CMP        #$58                 ; 'X'
                    BNE        ALL_PrepareStrUnk
*--     
ALL_PrepareStrX     PHX                             ; '%sX' : Copy String from address [X]
ALL_PrepareStrX_1   LDX        #$BBBB
                    JSR        ALL_InsertString     ; Add StringLength byte in the String
                    PLX
                    JMP        ALL_Prepare_1
*--     
ALL_PrepareStrY     PHX                             ; '%sY' : Copy String from address [Y]
ALL_PrepareStrY_1   LDX        #$CCCC
                    JSR        ALL_InsertString     ; Add StringLength bytes in the String
                    PLX
                    JMP        ALL_Prepare_1
*--
ALL_PrepareStrUnk   STA        ALL_LogBuffer+2,Y    ; Unknown Sequence %S sequence
                    LDA        #$25                 ; '%'
                    STA        ALL_LogBuffer,Y
                    LDA        #$53                 ; 'S'
                    STA        ALL_LogBuffer+1,Y
                    INY
                    INY
                    INY
                    JMP        ALL_Prepare_1                    
*----
ALL_PrepareEnd      LDA        #$0D                 ; Add the \r at the end of the Log Line
                    STA        ALL_LogBuffer,Y
                    INY
                    REP        #$30
                    TYA                             ; Compute Log Line Length
                    CLC
                    ADC        #$0014               ; Time/Date length = 20 bytes
                    STA        ALL_LogLength

*----  Write Line Log to LogFile.txt file  ----
ALL_Open            LDA        ALL_LogFilePath      ; File Path
                    STA        ALL_gsosOPEN+4
                    LDA        #$0002               ; Open access Mode = Write
                    STA        ALL_gsosOPEN+8     
                    JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       ALL_gsosOPEN         ; Parameters Table Address (Open in Read Mode)
                    BCS        ALL_ErrorEnd
                    LDA        ALL_gsosOPEN+2       ; File Descriptor
                    STA        ALL_gsosSETMARK+2
                    STA        ALL_gsosWRITE+2
                    STA        ALL_gsosCLOSE+2
*--
ALL_SetMark         LDA        #$0001               ; EOF
                    STA        ALL_gsosSETMARK+4
                    JSL        GSOS                 ; Set Mark to End
                    dw         $2016                ; Code for command 'SetMark'
                    adrl       ALL_gsosSETMARK      ; Parameters Table Address   
                    BCS        ALL_Error
*--
ALL_Write           LDA        #ALL_TimeBuffer      ; Time Data Buffer
                    STA        ALL_gsosWRITE+4
                    LDA        ALL_LogLength        ; Line Length = Time/Date + Log Line
                    STA        ALL_gsosWRITE+8
                    STZ        ALL_gsosWRITE+10
                    JSL        GSOS                 ; Write Time / Date
                    dw         $2013                ; Code for command 'Write'
                    adrl       ALL_gsosWRITE        ; Parameters Table Address   
                    BCS        ALL_Error
*--                 
ALL_Close           JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       ALL_gsosCLOSE        ; Parameters Table Address 
*--
ALL_End             RTS
*--
ALL_Error           JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       ALL_gsosCLOSE        ; Parameters Table Address 
                    SEC                             ; Set Error Flag
*--
ALL_ErrorEnd        RTS
               
*----------   Log File Variables   ---------     

ALL_LogFilePath     HEX        0000                 ; Log File Path Address
ALL_ParamValueHex   HEX        00000000             ; Hexadecimal Value copied from Address (Big Endian)
ALL_ParamValueAscii HEX        0000000000000000     ; Ascii Value (Big Endian)
ALL_LogLength       HEX        0000                 ; Log Buffer Length
ALL_TimeBuffer      ds         20                   ; 20 bytes for Ascii Time/Date
ALL_LogBuffer       ds         256                  ; Log Line, 256 char Max

TableHexAscii       ASC        '000102030405060708090A0B0C0D0E0F'
                    ASC        '101112131415161718191A1B1C1D1E1F'
                    ASC        '202122232425262728292A2B2C2D2E2F'
                    ASC        '303132333435363738393A3B3C3D3E3F'
                    ASC        '404142434445464748494A4B4C4D4E4F'
                    ASC        '505152535455565758595A5B5C5D5E5F'
                    ASC        '606162636465666768696A6B6C6D6E6F'
                    ASC        '707172737475767778797A7B7C7D7E7F'
                    ASC        '808182838485868788898A8B8C8D8E8F'
                    ASC        '909192939495969798999A9B9C9D9E9F'
                    ASC        'A0A1A2A3A4A5A6A7A8A9AAABACADAEAF'
                    ASC        'B0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF'
                    ASC        'C0C1C2C3C4C5C6C7C8C9CACBCCCDCECF'
                    ASC        'D0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF'
                    ASC        'E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEF'
                    ASC        'F0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF'

*----------   GS/OS Calls Variables   ---------
ALL_gsosOPEN        dw         3                    ; Number of parameters (Open in Write Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       LogFilePath          ; +4 File Pathname (init with an existing file path, so we only have to change 2 bytes)
                    ds         2                    ; +8 Request Access (1=Read,2=Write)

ALL_gsosSETMARK     dw         3                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
                    ds         2                    ; +4 Base
                    ds         4                    ; +6 Displacement

ALL_gsosWRITE       dw         5                    ; Number of parameters
                    ds         2                    ; +2  File Descriptor
                    adrl       LOG_LINE_1           ; +4  Data Buffer Address (init with an existing string path, so we only have to change 2 bytes)
                    ds         4                    ; +8  Data Size (Byte)
                    ds         4                    ; +12 Number of byte written to disk
                    ds         2                    ; +16 Cache Priority (#$0000 = No Cache)

ALL_gsosCLOSE       dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
                    
*----------   Insert Hexa Data into String   ---------
                    mx         %00
ALL_InsertHexaData1 PHX                             ; Copy 1 Byte in Hexa Ascii
*--
                    LDA        ALL_ParamValueHex    ; Byte #0
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
*--
                    PLX
                    RTS
*----
ALL_InsertHexaData2 PHX                             ; Copy 2 Bytes in Hexa Ascii
*--
                    LDA        ALL_ParamValueHex    ; Byte #0
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
*--
                    LDA        ALL_ParamValueHex+1  ; Byte #1
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY      
*--              
                    PLX
                    RTS
*----
ALL_InsertHexaData4 PHX                             ; Copy 4 Bytes in Hexa Ascii
*--
                    LDA        ALL_ParamValueHex    ; Byte #0
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
*--
                    LDA        ALL_ParamValueHex+1  ; Byte #1
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY 
*--     
                    LDA        ALL_ParamValueHex+2  ; Byte #2
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
*--  
                    LDA        ALL_ParamValueHex+3  ; Byte #3
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
*--          
                    PLX
                    RTS
                    
*----------   Insert SubString into String   ---------
                    mx          %10
ALL_InsertString    STX         ALL_InsertStr_1+1   ; Copy StringLength bytes
*-
                    LDX         #$0000
ALL_InsertStr_1     LDA         $DDDD,X
                    BEQ         ALL_InsertStringEnd
                    STA         ALL_LogBuffer,Y
                    INY
                    INX     
                    BRA         ALL_InsertStr_1
*-
ALL_InsertStringEnd RTS
                    
                    
********************************************************************************************************************
*  UnpackLZ4() :  Unpack LZ4 data buffer (IN:A=BankSrcDst,IN:X=OffsetSrcDst,IN:Y=PackDataSize,OUT:A=UnpackDataSize). *
********************************************************************************************************************
UnpackLZ4           ENT                             ; Uncompress a LZ4 Packed Data buffer (64 KB max)
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    STA        LZ4_Literal_3+1      ; A = Bank Src,Bank Dst
                    SEP        #$20                 
                    STA        LZ4_Match_5+1        
                    STA        LZ4_Match_5+2
                    XBA                             ;  => Return in A the length of unpacked Data
                    STA        LZ4_ReadToken+3  
                    STA        LZ4_Match_1+3    
                    STA        LZ4_GetLength_1+3
                    REP        #$30
                    TXA                             ; @Src LZ4 Data
                    AND        #$FF00
                    STA        LZ4_Init_1+1         
                    STA        LZ4_Init_2+1
                    TXA                             ; @Dst Uncompressed Data
                    XBA         
                    AND        #$FF00
                    STA        LZ4_DstOffset_1+1
                    STA        LZ4_DstOffset_2+1
                    TYA                             ; Y = Pack Data Size
                    CLC
LZ4_Init_1          ADC        #$AAAA               ; LZ4 Src Data offset
                    STA        LZ4_Limit+1          ; Last packed byte offset
LZ4_Init_2          LDA        #$AAAA               ; LZ4 Data
                    CLC
                    ADC        #$0010               ; LZ4 Header Size = 16 bytes
                    TAX                             ; X = First packed byte                 
*--
LZ4_DstOffset_1     LDY        #$8000               ; Init Target unpacked Data offset (#$0000 if dedicated uncompress buffer)
LZ4_ReadToken       LDAL       $AA0000,X            ; Read Token Byte
                    INX
                    STA        LZ4_Match_2+1
*-----------
LZ4_Literal         AND        #$00F0               ; >>> Process Literal Bytes <<<
                    BEQ        LZ4_Limit            ; No Literal
                    CMP        #$00F0
                    BNE        LZ4_Literal_1
                    JSR        LZ4_GetLengthLiter   ; Compute Literal Length with next bytes
                    BRA        LZ4_Literal_2
LZ4_Literal_1       LSR                             ; Literal Length use the 4 bit
                    LSR
                    LSR
                    LSR
*--
LZ4_Literal_2       DEC                             ; Copy A+1 Bytes
LZ4_Literal_3       MVN        $AA,$BB              ; Copy Literal Bytes from packed data buffer
                    PHK                             ; X and Y are auto incremented
                    PLB
*-----------
LZ4_Limit           CPX        #$AAAA               ; End Of Packed Data buffer ?
                    BEQ        LZ4_End
*-----------
LZ4_Match           TYA                             ; >>> Process Match Bytes <<<
                    SEC
LZ4_Match_1         SBCL       $AA0000,X            ; Match Offset
                    INX
                    INX
                    STA        LZ4_Match_4+1
*--
LZ4_Match_2         LDA        #$0000               ; Current Token Value
                    AND        #$000F
                    CMP        #$000F
                    BNE        LZ4_Match_3
                    JSR        LZ4_GetLengthMatch   ; Compute Match Length with next bytes
LZ4_Match_3         CLC
                    ADC        #$0003               ; Minimum Match Length is 4 (-1 for the MVN)
*--
                    PHX
LZ4_Match_4         LDX        #$AAAA               ; Match Byte Offset
LZ4_Match_5         MVN        $BB,$BB              ; Copy Match Bytes from unpacked data buffer
                    PHK                             ; X and Y are auto incremented
                    PLB
                    PLX
*-----------
                    BRA        LZ4_ReadToken
*-----------
LZ4_GetLengthLiter  LDA        #$000F               ; Compute Variable Length (Literal or Match)
LZ4_GetLengthMatch  STA        LZ4_GetLength_2+1
LZ4_GetLength_1     LDAL       $AA0000,X            ; Read Length Byte
                    INX
                    AND        #$00FF
                    CMP        #$00FF
                    BNE        LZ4_GetLength_3
                    CLC
LZ4_GetLength_2     ADC        #$000F
                    STA        LZ4_GetLength_2+1
                    BRA        LZ4_GetLength_1
LZ4_GetLength_3     ADC        LZ4_GetLength_2+1
                    RTS
*-----------
LZ4_End             TYA
                    SEC
LZ4_DstOffset_2     SBC        #$8000          ; A = Length of Unpack Data
                    PLB
                    RTL
                 
                 
**********************************************************
*  FadeOut() :  Fade Out Screen to black + Erase Screen. *
**********************************************************
FadeOut             ENT
                    LDY        #$000F               ; 16 Tones
*--
FOUT_1              LDX        #$01FE               ; 16 Palettes of 16 colors
*--
FOUT_2              LDAL       $019E00,X            ; Blue
                    AND        #$000F
                    BEQ        FOUT_3
                    LDAL       $019E00,X
                    SEC
                    SBC        #$0001
                    STAL       $019E00,X
FOUT_3              LDAL       $019E00,X            ; Green
                    AND        #$00F0
                    BEQ        FOUT_4
                    LDAL       $019E00,X
                    SEC
                    SBC        #$0010
                    STAL       $019E00,X
FOUT_4              LDAL       $019E00,X            ; Red
                    AND        #$0F00
                    BEQ        FOUT_5
                    LDAL       $019E00,X
                    SEC
                    SBC        #$0100
                    STAL       $019E00,X
*--
FOUT_5              DEX
                    DEX
                    BPL        FOUT_2
*--                 
FOUT_6              JSR        WaitForNextVBL       ; Tempo (1/60 sec)
*--
                    DEY
                    BPL        FOUT_1
*-----------
                    LDX        #$7FFE               ; Erase Screen
                    LDA        #$0000
FOUT_7              STAL       $012000,X
                    DEX
                    DEX
                    BPL        FOUT_7
*--         
                    RTL


**********************************************************************
*  FadeIN() :  Fade In Screen from Black to Picture (IN:A=@Palette). *
**********************************************************************
FadeIN              ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--                    
                    STA        FIN_5+1              ; @Palettes
                    STA        FIN_8+1
                    STA        FIN_11+1
*--
FIN_2               LDY        #$000F               ; 16 Tones
*--
FIN_3               LDX        #$01FE               ; 16 Palettes of 16 colors
*--
FIN_4               LDAL       $019E00,X            ; Blue
                    AND        #$000F
                    STA        FIN_6+1
FIN_5               LDA        $AAAA,X
                    AND        #$000F
FIN_6               CMP        #$AAAA
                    BEQ        FIN_7                    
                    LDAL       $019E00,X
                    CLC
                    ADC        #$0001
                    STAL       $019E00,X
*--
FIN_7               LDAL       $019E00,X            ; Green
                    AND        #$00F0
                    STA        FIN_9+1
FIN_8               LDA        $AAAA,X
                    AND        #$00F0
FIN_9               CMP        #$AAAA
                    BEQ        FIN_10
                    LDAL       $019E00,X
                    CLC
                    ADC        #$0010
                    STAL       $019E00,X
*--
FIN_10              LDAL       $019E00,X            ; Red
                    AND        #$0F00
                    STA        FIN_12+1
FIN_11              LDA        $AAAA,X
                    AND        #$0F00
FIN_12              CMP        #$AAAA
                    BEQ        FIN_13
                    LDAL       $019E00,X
                    CLC
                    ADC        #$0100
                    STAL       $019E00,X
*--
FIN_13              DEX
                    DEX
                    BPL        FIN_4
*--
FIN_14              JSR        WaitForNextVBL       ; Tempo (1/60 sec)
*-- 
                    DEY
                    BPL        FIN_3
                    PLB
                    RTL


**********************************************************
*  WaitForNextVBL() :  Wait for one VBL to be completed. *
**********************************************************
WaitForNextVBL      LDA        #$0096               ; Wait for the Spot at Line #150
                    JSL        WaitForSpot
WFNVBL_1            LDAL       RDVBLBAR-1           ; Wait For VBL
                    BPL        WFNVBL_1
                    RTS
                    
                    
********************************************************************************************
*  WaitForSpot() :  Wait for the Video Spot to cross a screen line (IN:A=Line[0000-00C7]). *
********************************************************************************************
WaitForSpot         ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--                             
                    LSR                             ; SHR Line # / 2 : 00-63
                    CLC
                    ADC        #$0080               ; C02E Line Code : 80-E3
                    SEP        #$20
                    STA        WFS_2+1
*--
WFS_1               LDAL       VERTCNT              ; Current Code [7D-FF]
WFS_2               CMP        #$AA                 ; Expected Line Code
                    BNE        WFS_1
*--
                    REP        #$30
                    PLB
                    RTL


************************************************************************************************
*  CheckForSpot() :  Check if the Video Spot is crossing a screen line (IN:A=Line[0000-00C7]). *
************************************************************************************************
CheckForSpot        LSR                             ; SHR Line # / 2 : 00-63
                    CLC
                    ADC        #$0080               ; C02E Line Code : 80-E3
                    SEP        #$20
                    STA        CFS_1+1
*--
                    LDAL       VERTCNT              ; Current Code [7D-FF]
CFS_1               CMP        #$AA                 ; Expected Line Code
                    BNE        CFS_NO
*--
CFS_YES             REP        #$30
                    LDA        #$0000
                    RTS
*--
CFS_NO              REP        #$30
                    LDA        #$0001
                    RTS
                  

**************************************************************************
*  ReadVideoSpot() :  Read the Video Spot current position (OUT:A=Line). *
**************************************************************************
ReadVideoSpot       ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--                             
                    SEP        #$20
                    LDAL       VERTCNT              ; Current Code [7D-FF]
                    STA        RVS_VERTCNT+1
                    LDAL       HORIZCNT
                    ASL                             ; Get Bit #7 into Carry
                    REP        #$30
RVS_VERTCNT         LDA        #$0000
                    ROL                             ; Move Carry into Bit #0
                    SEC
                    SBC        #$00FA               ; [00FA-01FF] -> [0000->0105] 
*--
                    PLB
                    RTL


****************************************************************
*  IncBorderColor() :  Increment Border Color (Debug Purpose). *
****************************************************************
IncBorderColor      ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--           
                    SEP        #$20
                    LDAL       CLOCKCTL             ; Current Border Color
                    STA        IncBorderColor_1+1
                    INC
                    AND        #$0F                 ; New Color
                    STA        IncBorderColor_2+1
IncBorderColor_1    LDA        #$AA
                    AND        #$F0
IncBorderColor_2    ORA        #$AA
                    STAL       CLOCKCTL             ; New Border Color
                    REP	       #$30
                    PLB
                    RTL

                    mx         %11                  ; Assemble in 8 bit

*****************************************************************
*  IncBorderColor8() :  Increment Border Color (Debug Purpose). *
*****************************************************************
IncBorderColor8     LDAL       CLOCKCTL             ; Current Border Color
                    STA        IncBorderColor8_1+1
                    INC
                    AND        #$0F                 ; New Color
                    STA        IncBorderColor8_2+1
IncBorderColor8_1   LDA        #$AA
                    AND        #$F0
IncBorderColor8_2   ORA        #$AA
                    STAL       CLOCKCTL             ; New Border Color
                    RTS

                    mx         %00                  ; Assemble in 16 bit

*******************************************************************
*  SetBorderColor() :  Increment Border Color (IN:A=BorderColor). *
*******************************************************************
SetBorderColor      ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    SEP        #$20
                    AND        #$0F
                    STA        SetBorderColor_1+1
                    LDAL       CLOCKCTL             ; Current Border Color
                    AND        #$F0
SetBorderColor_1    ORA        #$AA
                    STAL       CLOCKCTL             ; New Border Color
                    REP	       #$30
*--
                    PLB
                    RTL


******************************************************************
*  Set40ColTextPage1() :  Display Text Page 1 in 40 column mode. *
******************************************************************
Set40ColTextPage1  	SEP        #$20
                    LDA        #$01
                    STAL       NEWVIDEO             ; Display Text Page
                    STAL       CLR80VID             ; Enforce 40 column mode
                    REP	       #$30
                    RTS


*********************************************************
*  ClearTextPage1() :  Clear the Screen of Text Page 1. *
*********************************************************
ClearTextPage1      LDY        #$002E               ; 24 lines
CTP1_1              LDA        TEXTPAGELINEADDR,Y   ; Address of the Line
                    STA        CTP1_2+1
*--
                    LDX        #$0026               ; 40 characters wide
                    LDA        #$A0A0               ; Space x 2
CTP1_2              STAL       $E0AAAA,X            ; Page Text 1 is located in $E0/0400
                    DEX
                    DEX
                    BPL        CTP1_2
*--                    
                    DEY
                    DEY
                    BPL        CTP1_1
                    RTS


***************************************************************************
*  BuildTextScreen() :  Display Text information on Screen (IN:A=@Table). *
***************************************************************************
BuildTextScreen     STA        BTS_1+1              ; Table of Line Information Address
                    LDY        #$0000
*--
BTS_1               LDA        $AAAA,Y              ; @ Line Information
                    BEQ        BTS_End
                    STA        BTS_2+1              ; @ Y
                    INC
                    INC
                    STA        BTS_3+1              ; @ X
                    INC
                    INC
                    STA        BTS_4+1              ; @ String
*--                    
BTS_2               LDA        $AAAA                ; @ Y
                    ASL
                    TAX
                    LDA        TEXTPAGELINEADDR,X 
                    CLC
BTS_3               ADC        $AAAA                ; @ X
                    STA        BTS_5+1              ; Address in Text Page where to display the String
*--
                    SEP        #$30                 ; Display Line High-Ascii value
                    LDX        #$00
BTS_4               LDA        $AAAA,X              ; @ String
                    BEQ        BTS_6
BTS_5               STAL       $E0AAAA,X            ; Page Text 1 is located in $E0/0400
                    INX
                    BRA        BTS_4
BTS_6               REP        #$30
*--
                    INY                             ; Next Line
                    INY
                    BRA        BTS_1
*--                    
BTS_End             RTS

*-------------------------------- Misc Tables ----------------------------------
                    
TEXTPAGELINEADDR    DA         $0400,$0480,$0500,$0580,$0600,$0680,$0700,$0780
                    DA         $0428,$04A8,$0528,$05A8,$0628,$06A8,$0728,$07A8      
                    DA         $0450,$04D0,$0550,$05D0,$0650,$06D0,$0750,$07D0


****************************************************************
*  PauseGame() :  Pause Audio + Video waiting for a Key press. *
****************************************************************
PauseGame           ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--                    
                    JSL        EnableDisableAudio  ; Pause Audio
*--
                    LDX        #$1FE               ; Backup Palette
PG_BP               LDAL       $019E00,X
                    STA        PG_BufferPalette,X
                    DEX
                    DEX
                    BPL        PG_BP
*--
                    LDA        #$0000              ; For TAY 8 bits -> 16 bits
                    SEP        #$20
                    LDX        #$1FF               ; Dark Palette
PG_DP               LDAL       $019E00,X
                    TAY
                    LDA        PG_DarkPalette,Y    ; Divide by 2 RGB components
                    STAL       $019E00,X
                    DEX
                    BPL        PG_DP                    
                    REP        #$30
*--
                    JSL        WaitForKey          ; Wait for Key press
*--
                    LDX        #$1FE               ; Restaure Palette
PG_RP               LDA        PG_BufferPalette,X
                    STAL       $019E00,X
                    DEX
                    DEX
                    BPL        PG_RP
*--
                    JSL        EnableDisableAudio  ; Restart Audio
*--
                    PLB
                    RTL
                    
*----------   PauseGame Variables   ---------

PG_BufferPalette    DS         512                ; 16 Palettes

PG_DarkPalette      HEX        00,00,01,01,02,02,03,03,04,04,05,05,06,06,07,07
                    HEX        00,00,01,01,02,02,03,03,04,04,05,05,06,06,07,07
                    HEX        10,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17
                    HEX        10,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17
                    HEX        20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27
                    HEX        20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27
                    HEX        30,30,31,31,32,32,33,33,34,34,35,35,36,36,37,37
                    HEX        30,30,31,31,32,32,33,33,34,34,35,35,36,36,37,37
                    HEX        40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
                    HEX        40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
                    HEX        50,50,51,51,52,52,53,53,54,54,55,55,56,56,57,57
                    HEX        50,50,51,51,52,52,53,53,54,54,55,55,56,56,57,57
                    HEX        60,60,61,61,62,62,63,63,64,64,65,65,66,66,67,67
                    HEX        60,60,61,61,62,62,63,63,64,64,65,65,66,66,67,67
                    HEX        70,70,71,71,72,72,73,73,74,74,75,75,76,76,77,77
                    HEX        70,70,71,71,72,72,73,73,74,74,75,75,76,76,77,77


******************************************************************************************
*  HexDec2Hex() :  Convert a Decimal number into Hexadecimal (IN:A=NumDec,OUT:A=NumHex). *
******************************************************************************************
HexDec2Hex          STA        HD2H_D+1             ; Convert 32 as 3*10+2 = 1E + 2 = 20
                    AND        #$000F
                    STA        HD2H_U+1
*--
HD2H_D              LDA        #$AAAA
                    AND        #$00F0
                    LSR
                    LSR
                    LSR
                    LSR
                    TAX
                    LDA        HD2H_Tab10,X         ; Dozen
                    AND        #$00FF
*--
                    CLC
HD2H_U              ADC        #$AAAA               ; Unit
*--
                    RTS                    

*----------   HexDec2Hex Variables   ---------
HD2H_Tab10          HEX        00,0A,14,1E,28,32,3C,46,50,5A,64,6E,78,82,8C,96          


******************************************************************************************
*  Hex2HexDec() :  Convert a Hexadecimal number into Decimal (IN:A=NumHex,OUT:A=NumDec). *
******************************************************************************************
Hex2HexDec          AND        #$00FF               ; Convert $2A as 2*16+10 = 42 = $42
                    TAX
                    LDA        H2HD_Tab,X
                    AND        #$00FF
                    RTS                    

*----------   HexDec2Hex Variables   ---------
H2HD_Tab            HEX        00,01,02,03,04,05,06,07,08,09,10,11,12,13,14,15
                    HEX        16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
                    HEX        32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
                    HEX        48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63
                    HEX        64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79
                    HEX        80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95
                    HEX        96,97,98,99          


*******************************************************************
*  HBEntryPoint() :  HeartBeat Interruption Entry Point function. *
*******************************************************************
HBEntryPoint        ds         4
                    dw         1                    ; 1/60 seconds
                    dw         $A55A                ; Signature
*--
                    CLC
                    XCE
                    REP        #$30
*--
                    LDA        #$0001               ; Reset Timer
                    STA        HBEntryPoint+4
*--
                    LDAL       HBCounter            ; Increment Counter
                    INC
                    STAL       HBCounter
*--
                    SEP        #$30
                    CLC
                    RTL
                    mx         %00
                    
                     
*****************************************************************
*  HBResetCounter() :  Reset Counter of HeartBeat Interruption. *
*****************************************************************
HBResetCounter      STZ        HBCounter            ; Set VBL counter to 0
                    STZ        HBCounter
                    STZ        HBCounter 
                    STZ        HBCounter 
                    RTS


********************************************************************************
*  HBGetCounter() :  Get HeartBeat Interruption Counter value (OUT:A=Counter). *
********************************************************************************
HBGetCounter        LDA        HBCounter            ; Get VBL counter
                    RTS

*----------   HBEntryPoint / HBResetCounter / HBGetCounter Variables   ---------
HBCounter           HEX        0000
                          

******************************************************************************
*  InitRandomNumber() :  Init Index for a random number (OUT:A=Index[0-99]). *
******************************************************************************
InitRandomNumber    ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    LDAL       VERTCNT              ; Current Code [7D-FF]
                    AND        #$00FF
                    SEC
                    SBC        #$007D 
                    CMP        #$0064
                    BLT        IRN_End
                    SEC
                    SBC        #$0064               ; Keep value in range [$00-$63]
*--
IRN_End             PLB
                    RTL
                    
                    
*****************************************************************************************
*  GetRandomNumber() :  Get a random number between [1-100] (IN:A=@Index,OUT:A=Number). *
*****************************************************************************************
GetRandomNumber     ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
                    TAX                             ; Index Address
                    LDA        $04,S                ; Get Caller's Bank  A=??BB
                    AND        #$00FF
                    STA        GRN_GetIndex+2       ; Index Bank
                    STA        GRN_SetIndex+2
GRN_GetIndex        LDAL       $BB0000,X
                    INC
                    CMP        #$0064               ; Maintains Index into range [0-99]
                    BNE        GRN_SetIndex
                    LDA        #$0000
GRN_SetIndex        STAL       $BB0000,X
                    TAX
                    LDA        GRN_Table,X
                    AND        #$00FF
                    PLB
                    RTL
                    
*----------   GetRandomNumber Variables   ---------
GRN_Table           HEX        49,4E,16,38,5F,33,48,20,4A,45,0E,64,40,1F,04,12,28,3A,34,5B
                    HEX        43,30,23,4C,2E,21,5D,3E,06,2A,0D,22,3B,58,4F,50,60,47,27,11
                    HEX        25,1D,39,0F,44,10,01,0A,2B,57,18,55,53,2F,15,63,31,35,1B,1A
                    HEX        1C,3C,3F,02,0C,61,17,2D,51,05,59,13,36,46,09,52,5C,56,42,5A
                    HEX        24,54,29,32,5E,41,62,19,4D,1E,26,37,07,0B,03,2C,4B,3D,08,14
                             
*----------  Global Tables                             
SHR_LINE_TABLE      HEX        0000,A000,4001,E001,8002,2003,C003,6004,0005,A005,4006,E006,8007,2008,C008,6009,000A,A00A,400B,E00B
                    HEX        800C,200D,C00D,600E,000F,A00F,4010,E010,8011,2012,C012,6013,0014,A014,4015,E015,8016,2017,C017,6018
                    HEX        0019,A019,401A,E01A,801B,201C,C01C,601D,001E,A01E,401F,E01F,8020,2021,C021,6022,0023,A023,4024,E024
                    HEX        8025,2026,C026,6027,0028,A028,4029,E029,802A,202B,C02B,602C,002D,A02D,402E,E02E,802F,2030,C030,6031
                    HEX        0032,A032,4033,E033,8034,2035,C035,6036,0037,A037,4038,E038,8039,203A,C03A,603B,003C,A03C,403D,E03D
                    HEX        803E,203F,C03F,6040,0041,A041,4042,E042,8043,2044,C044,6045,0046,A046,4047,E047,8048,2049,C049,604A
                    HEX        004B,A04B,404C,E04C,804D,204E,C04E,604F,0050,A050,4051,E051,8052,2053,C053,6054,0055,A055,4056,E056
                    HEX        8057,2058,C058,6059,005A,A05A,405B,E05B,805C,205D,C05D,605E,005F,A05F,4060,E060,8061,2062,C062,6063
                    HEX        0064,A064,4065,E065,8066,2067,C067,6068,0069,A069,406A,E06A,806B,206C,C06C,606D,006E,A06E,406F,E06F
                    HEX        8070,2071,C071,6072,0073,A073,4074,E074,8075,2076,C076,6077,0078,A078,4079,E079,807A,207B,C07B,607C
                             
********************************************************************************
