***********************************************************************************************
*                                                                                             *
*  AppleIIgsKarate.s :  Apple IIgs version of ST|Amiga Karate / (c) 1986 Paradox / Eidersoft. *
*                                                                                             *
***********************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI   *   Brutal Deluxe Software   *   April 2022   *
***********************************************************************************************

*----------   Global Equates : Firmware / Hardware / GS/OS   ---------
KBDREAD             =          $00C000              ; Read Keyboard
CLR80VID            =          $00C00C              ; Disable 80 Column display
SET80VID            =          $00C00D              ; Enable 80 Column display
KBDSTRB             =          $00C010              ; Keyboard Strobe
RDVBLBAR            =          $00C019              ; VBL Status (Bit #7=0 if VBL)
RD80VID             =          $00C01F              ; 80 column status (Bit #7=1 if 80 column)
TBCOLOR             =          $00C022              ; Text / Background color
KEYMODREG           =          $00C025              ; Special Keys (Shift, Control, Apple...)
NEWVIDEO            =          $00C029              ; Apple IIgs Video Mode
VERTCNT             =          $00C02E              ; Video Controler Vertical Counter
HORIZCNT            =          $00C02F              ; Video Controler Horizontal Counter
CLOCKCTL            =          $00C034              ; Border Color + Clock
SHADOW              =          $00C035              ; Shadowing Register
SOUNDCTL            =          $E1C03C              ; Ensoniq 5503 Control Register
SOUNDDATA           =          $E1C03D              ; Ensoniq 5503 Data Register
SOUNDADRL           =          $E1C03E              ; Ensoniq 5503 Address Low
SOUNDADRH           =          $E1C03F              ; Ensoniq 5503 Address High
BUTN0               =          $E0C061              ; Open Apple Key = Joystick Button 0
BUTN1               =          $E0C062              ; Open Apple Key = Joystick Button 1
CYAREG              =          $E0C036              ; Bit 7=Fast[1]/Slow[0]
PADDL0              =          $E0C064              ; Paddle(0) = X Axis
PADDL1              =          $E0C065              ; Paddle(1) = Y Axis
PTRIG               =          $E0C070              ; Paddle Trigger
TEXTPAGE1           =          $E00400              ; Text Page #1 $0400-$07FF
GSOS                =          $E100A8              ; GS/OS Entry Point
CTRLPNLSNDVOL       =          $E100CA              ; Control Panel Sound Volume


*----------   Merlin 16+ Directives   ---------

                    mx         %00                  ; Assemble in 16 bit

                    rel                             ; Build a relocated S16 file
                    dsk        Main.l               ; Name of the Segment

                    use        4/Locator.Macs       ; Macro Definition Files
                    use        4/Mem.Macs
                    use        4/Misc.Macs
                    use        4/Util.Macs
                    use        4/Sound.Macs


*----------   External Labels from AppleIIgsKarate.Game.s Segment  ---------
PlayGame            EXT                             ; Game Function Entry Point
LoadGameSound       EXT                             ; Load + Install Game Sounds
PG_Score            EXT                             ; Player Score
PG_Level            EXT                             ; Game Level
GameBankTable       EXT                             ; Memory Bank Table

*----------   External Labels from AppleIIgsKarate.Sprite*.s Segment  ---------
DrawSpriteBank1     EXT                             ; Compiled Sprites Bank #1
DrawSpriteBank2     EXT                             ; Compiled Sprites Bank #2
DrawSpriteBank3     EXT                             ; Compiled Sprites Bank #3
DrawSpriteBank4     EXT                             ; Compiled Sprites Bank #4


**********************************************
*  Main() :  Entry point of the Application. *
********************************************** 

Main                PHK                             ; Data Bank Register = Program Bank Register
                    PLB
                    SEP        #$20
                    PHK                             ; Save Current Bank
                    PLA
                    STA        CurrentBank+1        ; XX 
                                        
                    CLC                             ; 16 bit
                    XCE
                    REP        #$30

*-----   Program Initialization   ---------

M_Init              JSR        ToolInitialize       ; Init Tools + Compact Memory + Ask for Shadowing
                    JSR        BackupEnvironment    ; Backup environment (colors...)
                    JSR        ComputeGSSpeed       ; Check the Apple IIgs performance


*-----   Allocate Memory + Start HeartBeat 1/60s Interruptions

M_Alloc             JSR        AllocProgramMemory   ; Memory Allocation
                    BCC        M_HeartBeat
                    JMP        Error_Memory

M_HeartBeat         SEI                             ; Enable HeartBeat
                    PushLong   #HBEntryPoint
                    _SetHeartBeat
                    CLI
                    INC        M_End+1              ; HeartBeat is set
                    

*-----   Boot Screen : Display SHR + Display Logo + Text + Sound + Load next screen files  ----------

M_BootScreen        JSR        BootScreen           ; Graphic + Logo + Text + Sound + Load Files
                    BEQ        M_CreditScreen
                    JMP        Error_LoadFile       ; A = DataFile ID
                    
                            
*-----   Credit Screen : Display Credit Screen + Play Music + Load next screen files   ------

M_CreditScreen      JSR        CreditScreen         ; Display Credit + Play Music + Load Files
                    BEQ        M_SettingsScreen
                    JMP        Error_LoadFile       ; A = DataFile ID
                    
*-----   SettingsScreen : Display Game Settings   --------

M_SettingsScreen    JSR        SettingsScreen       ; Display/Handle Settings
*--
M_SettingsAction    CMP        #$0051               ; 'Q' = Quit
                    BEQ        M_End
                    CMP        #$0030               ; '0' = Play New Game
                    BEQ        M_GameScreen
                    CMP        #$0048               ; 'H' = High Score
                    BEQ        M_HighScoreScreen

*-----   HighScore : Display High Score Table   --------

M_HighScoreScreen   LDA        #$0000               ; Display High Score Table
                    JSR        HighScoreScreen
*--
                    BRA        M_SettingsScreen     ; Go Back to SettingsScreen

*-----   Game Sequence : Play New Game   --------

M_GameScreen        JSL        PlayGame             ; Play Game
                    CMP        #$009B               ; Esc : Stop Game
                    BEQ        M_SettingsScreen
                    CMP        #$FFFF               ; Fatal Error
                    BEQ        M_End
*--
                    BRA        M_SettingsScreen     ; Back To Setting Menu

*-----   End Of Program   ---------

M_End               LDA        #$0000               ; Set to 1 if HeartBeat previously registered
                    BEQ        M_End_1
*--                    
                    SEI                             ; Delete HeartBeat
                    PushLong   #HBEntryPoint
                    _DelHeartBeat
                    CLI
*--              
M_End_1             JSR        ClearTextPage1       ; Clear Page Text #1
                    JSR        RestoreEnvironment   ; Restore environment (colors...)

M_End_2             JSR        ToolTerminate        ; End up Tools
                    JMP        ExitToLauncher       ; Quit to the Launcher


*********************************************************************************
*  Error_XXX() :  Error Handling Routine (Memory allocation, File access, ...). *
*********************************************************************************
Error_Memory        LDX        #ERR_LineMemory1     ; Memory Allocation Error
                    LDY        #ERR_LineMemory2
                    BRA        ErrorHandling
*----
Error_LoadFile      TAX                             ; A = File Path STRL
                    LDA:       $0000,X              ; File Path Length
                    STA        Error_LoadFile_2+1
                    SEP        #$20                 ; Copy File Path
                    LDY        #$0000
Error_LoadFile_1    LDA:       $0002,X
                    ORA        #$80
                    STA        ERR_LineLoadFile2+12,Y
                    INX
                    INY
Error_LoadFile_2    CPY        #$AAAA
                    BNE        Error_LoadFile_1
                    REP        #$30        
                    LDX        #ERR_LineLoadFile1   ; Can't Load Data File
                    LDY        #ERR_LineLoadFile2
                    BRA        ErrorHandling
*----                    
ErrorHandling       STX        Error_TabLine+2      ; Install Error Lines
                    STY        Error_TabLine+4
                    JSR        Set40ColTextPage1    ; Display Text Page #1 / 40 Column                    
                    JSR        ClearTextPage1       ; Clear Page Text #1
                    LDA        BE_C022              ; Restaure Border Color
                    AND        #$00FF
                    JSL        SetBorderColor
                    LDA        #Error_TabLine
                    JSR        BuildTextScreen      ; Display Text Error message
*--
                    JSL        WaitForKey           ; Wait for Key Press
*--
                    JMP        M_End                ; End Program

*--------------   Error Screen Variables   -------------
Error_TabLine       DA         ERR_Line1,ERR_Line2,ERR_Line2,ERR_Line3
                    HEX        0000

ERR_Line1           HEX        0400,0000            ; Y=4, X=0
                    ASC        "      A Fatal Error Has Occurred !      ",00
ERR_Line2           HEX        1000,0000            ; Y=16, X=0
                    ASC        "                                        ",00
ERR_Line3           HEX        1600,0000            ; Y=22, X=0
                    ASC        "       - Press Any Key to Exit -        ",00
*--
ERR_LineMemory1     HEX        0F00,0000            ; Y=15, X=0
                    ASC        "  Error, Can't allocate enough memory.  ",00
ERR_LineMemory2     HEX        1000,0000            ; Y=16, X=0
                    ASC        "This program requires 832 KB of free RAM",00
*--
ERR_LineLoadFile1   HEX        0A00,0000            ; Y=8, X=0
                    ASC        " Error, Can't load Data File from Disk :",00
ERR_LineLoadFile2   HEX        0D00,0000            ; Y=10, X=0
                    ASC        "                                        ",00


************************************************************************************
*  AllocProgramMemory() :  Allocate Memory for the Program (OUT:Carry=SetOnError). *
************************************************************************************
AllocProgramMemory  JSR        AllocOneBank         ; Memory Allocation for Load/Unpack Bank
                    BMI        APM_Error
                    STA        LoadUnpackBank
*--
                    JSR        AllocOneBank         ; Memory Allocation for Generic Data Bank x 3
                    BMI        APM_Error
                    STA        Data1Bank
                    JSR        AllocOneBank
                    BMI        APM_Error
                    STA        Data2Bank
                    JSR        AllocOneBank
                    BMI        APM_Error
                    STA        Data3Bank      
*-
                    JSR        AllocOneBank         ; Memory Allocation for Erase Code Bank x 3
                    BMI        APM_Error
                    STA        EraseCode1Bank
                    JSR        AllocOneBank
                    BMI        APM_Error
                    STA        EraseCode2Bank
                    JSR        AllocOneBank
                    BMI        APM_Error
                    STA        EraseCode3Bank      
*----
                    LDX        #$0000               ; Copy Allocated Banks to Game Segment
APM_Copy            LDA        LoadUnpackBank,X
                    STAL       GameBankTable,X
                    INX
                    INX
                    CPX        #$000E
                    BNE        APM_Copy
*----
                    LDA        CurrentBank     
*----
APM_End             CLC                             ; OK
                    RTS  
*--
APM_Error           SEC                             ; Error
                    RTS  

*--------  AllocProgramMemory variables  --------
CurrentBank         HEX        0000                 ; Program Current Bank 00XX
LoadUnpackBank      HEX        0000                 ; Load / Unpack Bank   00XX (0000-7FFF : Load / 8000-FFFF : Unpack)
Data1Bank           HEX        0000                 ; Generic Data Bank #1 + LZ4 Game Background Data
Data2Bank           HEX        0000                 ; Generic Data Bank #2
Data3Bank           HEX        0000                 ; Generic Data Bank #3
*-
EraseCode1Bank      HEX        0000                 ; Erase Code Bank #1
EraseCode2Bank      HEX        0000                 ; Erase Code Bank #2
EraseCode3Bank      HEX        0000                 ; Erase Code Bank #3
*-----
StackAddress        HEX        0000                 ; Stack Address Backup


*-------------------------- Global Buffer --------------------------------------

Buffer_Palette      ENT
                    DS         512                  ; 16 Palettes

*------------------------ Other Source Files -----------------------------------

*--  Screen Functions
                    PUT        AppleIIgsKarate.Screen  ; Screen Functions

*--  Shared Functions
                    PUT        AppleIIgsKarate.Shared  ; Generic Functions Library
                    
*--  Graphic Functions 
                    PUT        AppleIIgsKarate.Graphic ; Bitmap Graphics Functions

*--  Sound Functions
                    PUT        AppleIIgsKarate.Sound   ; Sound Effects Functions
                                    
                    ASC        '- End of File -'

********************************************************************************
